c**********************************************************************
c
c                       ================
c                       PROGRAM : COMBAS
c                       ================
c
c       This program combines the basis set contained in several index
c       files and makes unified basis set.  It can be used to avoid
c       getting the pruned basis repeatedly for MOMD (varying director
c       tilt angles) or NLS (nonlinear least square fitting) type
c       calculations.
c
c       Includes:
c               stddim.inc
c               stdio.inc
c               eprdat.inc
c               indexf.inc
c               fnames.inc
c
c       Uses:
c               getids.f
c               setnam.f
c               rdfdat.f
c               wrfdat.f
c
c**********************************************************************
c
      program combas
c
      implicit none
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'indexf.inc'
      include 'fnames.inc'
c
      logical fexist
      integer i,j,maxle,maxlo,maxk,maxm,maxpi,minle,minlo,
     #      mink,minm,minpi,n,nfiles,nbas,crdim,indim,picr,
     #      qicr,lcr,jkcr,kcr,jmcr,mcr,ipar,ierr,ncalc    
c
      integer jqe2(mxdim),pi2(mxdim),qi2(mxdim),l2(mxdim),
     #        jk2(mxdim),k2(mxdim),jm2(mxdim),m2(mxdim)
c
      external ipar
c
c######################################################################
c
      write(luttyo,1000)
      write(luttyo,1010)
      n=1
 15   write (luttyo,1020)
      read (luttyi,1025,err=15) files(1)
 20   n=n+1
 25   write (luttyo,1030)
      read (luttyi,1025,err=25) files(n)
      if (files(n).eq.' ') then
        n=n-1
        go to 30
      end if
      if (n.lt.mxcalc) go to 20
c
 30   continue
      crdim=0
c
c======================================================================
c     loop over calculations
c======================================================================
c
      do 900 ncalc=2,n
c
c---------------------------------------------------------------------
c     construct file names
c---------------------------------------------------------------------
c
      fileid = files(ncalc)
      ips=1
      call setnam
c
c----------------------------------------------------------------------
c     read parameters from parameter file
c----------------------------------------------------------------------
c
      inquire(file=prname,exist=fexist)
c
      if (fexist) then 
        call rdfdat(prname,ierr)
      else
	write (luttyo,1100) prname(:namlth)
        stop
      endif
c
c----------------------------------------------------------------------
c     Read basis set information from index file
c----------------------------------------------------------------------
c
      inquire(file=ixname,exist=fexist)
      if (fexist) then
        open (unit=ludisk,file=ixname,status='old',
     #        access='sequential',form='unformatted')
        read (ludisk) ndim
        read (ludisk) (pi2(i),qi2(i),l2(i),jk2(i),k2(i),
     #                 jm2(i),m2(i),i=1,ndim)
        close (unit=ludisk)
      else
        write (luttyo,1110)
        stop
      end if
c
      indim=0
      if (ncalc.eq.2) then
c			*** copy first basis set ***
        crdim=ndim
        do 40 i=1,ndim
          pi1(i)=pi2(i)
          qi1(i)=qi2(i)
          l1(i)=l2(i)
          jk1(i)=jk2(i)
          k1(i)=k2(i)
          jm1(i)=jm2(i)
          m1(i)=m2(i)
 40     continue
      else
c
c----------------------------------------------------------------------
c       Go through the basis set if it has new basis.
c----------------------------------------------------------------------
c
        do 50 i=1,ndim
          picr=pi2(i)
          qicr=qi2(i)
          lcr=l2(i)
          jkcr=jk2(i)
          kcr=k2(i)
          jmcr=jm2(i)
          mcr=m2(i)
c
          do 60 j=1,crdim
            if ((picr.eq.pi1(j)).and.(qicr.eq.qi1(j)).and.
     #          (lcr.eq.l1(j)).and.(jkcr.eq.jk1(j)).and.
     #          (kcr.eq.k1(j)).and.(jmcr.eq.jm1(j)).and.
     #          (mcr.eq.m1(j)))  go to 50
 60       continue
c
          indim=indim+1
          pi1(crdim+indim)=picr
          qi1(crdim+indim)=qicr
          l1(crdim+indim)=lcr
          jk1(crdim+indim)=jkcr
          k1(crdim+indim)=kcr
          jm1(crdim+indim)=jmcr
          m1(crdim+indim)=mcr
 50     continue
c
      end if
c
      write (luttyo,1200) crdim,crdim+indim
      crdim=crdim+indim
c
 900  continue
c
c----------------------------------------------------------------------
c     loop over basis set data
c----------------------------------------------------------------------
c                                      
      maxle=-1
      minle=1000
      maxlo=-1
      minlo=1000
      maxk=-1
      mink=1000
      maxm=-1000
      minm=1000
      maxpi=-1000
      minpi=1000
c
      do 200 j=1,crdim
c
        n=l1(j)
        if (ipar(n).eq.1) then
          if (n.gt.maxle) maxle=n
          if (n.lt.minle) minle=n
        else
          if (n.gt.maxlo) maxlo=n
          if (n.lt.minlo) minlo=n
        end if
c
        n=k1(j)
        if (n.gt.maxk) maxk=n
        if (n.lt.mink) mink=n
c
        n=m1(j)
        if (n.gt.maxm) maxm=n
        if (n.lt.minm) minm=n
c
        n=pi1(j)
        if (n.gt.maxpi) maxpi=n
        if (n.lt.minpi) minpi=n
 200  continue
c
      if (maxlo.eq.-1) maxlo=0
      if (minlo.eq.1000) minlo=0
      write(luttyo,1210)
      write(luttyo,1220) ptol,minle,maxle,minlo,maxlo,mink,maxk,
     #     minm,maxm,minpi,maxpi,crdim
c
      write (luttyo,1230)
c
      ndim=crdim
      lemx=maxle
      lomx=maxlo
      kmx=maxk
      mmx=maxm
      ipnmx=maxpi
c
      fileid = files(1)
      ips=1
      call setnam
c
      open (unit=ludisk,file=ixname,status='unknown',
     #      access='sequential',form='unformatted')
      write (ludisk) ndim
      write (ludisk) (pi1(i),qi1(i),l1(i),jk1(i),k1(i),
     #               jm1(i),m1(i),i=1,ndim)
      close (unit=ludisk)
c
      call wrfdat(prname)
c
c----------------------------------------------------------------------
c     exit from program
c----------------------------------------------------------------------
c
      write(luttyo,1000)
      stop
c
c======================================================================
c    format statements
c======================================================================
c
 1000 format(//,2x,70('#'),//)
 1010 format(28x,'PROGRAM : COMBAS',/,28x,'---------------',/)
 1020 format(2x,'Enter Index fileid (output) without extension : ',$)
 1030 format(2x,'Enter Index fileid (input) without extension ',
     #       '(<CR> to end): ',$)
 1025 format(a)
 1100 format(/,20x,'*** CANNOT FIND FILE ',a,' ***',/)
 1110 format(/,10x,'!!! INDEX FILE DOES NOT EXIST !!!')
 1200 format(/,2x,'old dimension = ',i4,5x,'new dimension = ',i5)
 1210 format(//2x,70('-')/
     #       3x,'error',7x,'Le',8x,'Lo',8x,'K',9x,'M',9x,'pI',
     #       /,12x,5('min  max  '),2x,'ndim'/ )
 1220 format(g10.3,10(i5),i6)
 1230 format(/,2x,'writing files : <id>.parx, <id>.indx')
c
c======================================================================
c
      end
