c*********************************************************************
c
c                       =================
c                       SUBROUTINE COMGAP
c                       =================
c
c     Subroutine COMGAP computes the minimal gaps between
c     distinct T-eigenvalues supplied.
c
c     on return :
c        |gg(k)| = min(j.ne.k,|vs(k)-vs(j)|)
c        mp2(k)=j index for minimum.
c
c        gg(k) < 0 means nearest neighbor is spurious.
c
c     includes :
c
c        stddim.inc
c        stdio.inc
c
c*********************************************************************
c
      subroutine comgap(vc,va,gg,mp,ind,m,igap,itag)
c
      implicit none
c
      include 'stddim.inc'
      include 'stdio.inc'
c
      complex*16 vc(mxstep),z
      double precision  va(mxstep),t0,t1,tu,tk
      real  gg(mxstep),gtemp
      integer  igap,itag,index,itemp,j,k,kk,km1,kp1,l,m
      integer  mp(mxstep),ind(mxstep)
c
c---------------------------------------------------------------------
c     if igap = 0 we do not order eigenvalues by increasing gap size
c
c     va(k) = |vc(k)|  va(k) <= va(k+1)
c     gg(k) = min |vc(k)-vc(j)|  j .ne. k.
c---------------------------------------------------------------------
c
      tu=va(m)+va(m)
      k=0
 10   k=k+1
      if (k.gt.m) go to 60
      index=0
      t1=tu
      tk=va(k)
      z=vc(k)
      j=k
c     backwards
 20   j=j-1
      if (j.lt.1) go to 30
      t0=tk-va(j)
      if (t0.gt.t1) go to 30
      t0=cdabs(z-vc(j))
      if (t1.le.t0) go to 20
      t1=t0
      index=j
      go to 20
c     forwards
 30   j=k
 40   j=j+1
      if (j.gt.m) go to 50
      t0=va(j)-tk
      if (t0.gt.t1) go to 50
      t0=cdabs(z-vc(j))
      if (t1.le.t0) go to 40
      t1=t0
      index=j
      go to 40
 50   ind(k)=index
      gg(k)=t1
      if(itag.eq.0) go to 10
      if(mp(index).eq.0)  gg(k)=-gg(k)
      go to 10
c
 60   continue
c
      if (igap.eq.0) go to 140
c
      write(luttyo,70)
c
      write(luttyo,80) (k,ind(k),vc(k),gg(k), k=1,m)
c
c     order vc g by increasing mingap size
      do 90 j=1,m
      ind(j)=j
 90   continue
c
      do 110 k=2,m
      km1=k-1
      do 100 l=1,km1
      kk=k-l
      kp1=kk+1
      if (abs(gg(kp1)).ge.abs(gg(kk))) go to 110
      z=vc(kk)
      vc(kk)=vc(kp1)
      vc(kp1)=z
      gtemp=gg(kk)
      gg(kk)=gg(kp1)
      gg(kp1)=gtemp
      itemp=ind(kk)
      ind(kk)=ind(kp1)
      ind(kp1)=itemp
 100  continue
 110  continue
c
      write(luttyo,120)
c
      write(luttyo,80) (k,ind(k),vc(k),gg(k), k=1,m)
c
 140  continue
      return
c
c=====================================================================
c     format statements
c=====================================================================
c
 70   format(' mingaps for good T-eigenvalues in COMGAP'/
     # 1x,'evnum',1x,'neigh',9x,'Re(ev)',11x,'Im(ev)',9x,'mingap')
 80   format(2i6,5x,g14.7,3x,g14.7,3x,g9.3)
 120  format(' T-eigenvalues ordered by increasing mingap'/
     # 1x,'gapnum',1x,'evnum',9x,'Re(ev)',11x,'Im(ev)',9x,'mingap')
      end
