c*********************************************************************
c
c                       ================
c                       SUBROUTINE CSVAL
c                       ================
c
c     This subroutine calculates the distinct eigenvalues of a given
c     tridiagonal matrix.  The identification test for good versus
c     spurious eigenvalues is done according to the procedure in
c     Cullum & Willoughby's book.
c
c       Arguments:
c           alpha    diagonal T matrix elements
c           beta     extradiagonal T matrix elements
c           v1       work space
c           vs       work space
c           v2       work space, on exit = eigenvalues of T-matrix sorted
c                    in the order of decreasing weighting factor
c           w        weighting factor
c           ngood    # of good eigenvalues (passed via eprdat.inc common)
c           ndiag    dimension of T-matrix used in the calculation of the
c                    eigenvalues (passed via eprdat.inc common)
c           nstep    dimension of T-matrix generated in Lanczos routine
c
c     modified from CSLEVAL.F by SHL 17-AUG-92
c
c     Includes :
c               stddim.inc
c               stdio.inc
c               eprdat.inc
c               fnames.inc
c               rndoff.inc
c     Uses :
c               compev.f
c               lump.f
c               comgap.f
c               isoev.f
c               inverr.f
c
c*********************************************************************
c
      subroutine csval(alpha,beta,v1,v2,vs,w)
c
      implicit none
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'fnames.inc'
      include 'rndoff.inc'
c
      integer  mxinit,iconv,mev,ndis,loop,igap,itag
      integer  i,j,niso,it,iwrite,l,m,mmb,ierr
      integer  mp(mxstep),mp2(mxstep)
c
      real  g(mxstep),gg(mxstep),gtp,ggtp
c
      double precision  gr(mxstep),gc(mxstep)
      double precision  gaptol,ttol,epsm,reltol,evmax,t0,t1
      double precision  scale1,scale2,sputol,contol,multol
      double precision  one,zero
      parameter (one=1.0D0,zero=0.0D0)
c
      complex*16  alpha(mxstep),beta(mxstep),vs(mxstep),w(mxstep)
      complex*16  v1(mxstep),v2(mxstep)
      complex*16  betam,wt,egv
      complex*16  czero,ci
      parameter (czero=(0.0D0,0.0D0),ci=(0.0D0,1.0D0))
c
c#####################################################################
c
      if (tstflg) go to 99
c
c---------------------------------------------------------------------
c     Eigenvalue calculation without test for spurious eigenvalues
c---------------------------------------------------------------------
c
      do 82 i=1,nstep
        v2(i)=alpha(i)
        v1(i)=beta(i)
 82   continue
c
      call cmtqli(nstep,v2,v1,w,1,ierr)
c
      if (ierr.ne.0) then
        write(luttyo,80)
        go to 800
      end if
c
      ngood=nstep
      ndiag=nstep
c
      return
c
c---------------------------------------------------------------------
c     Eigenvalue calculation with test for spurious eigenvalues
c---------------------------------------------------------------------
c
 99   epsm=100.0D0*rndoff
      scale1=5.0D2
      scale2=5.0D0
      gaptol=1.0D-7
      reltol=1.0D-8
      mxinit=10      
      iconv=0
      mev=0
      mmb=0
c
c---------------------------------------------------------------------
c     loop on the size of the T-matrix
c---------------------------------------------------------------------
c
      if (recflg) then
          open (unit=ludisk,file=dename,status='unknown',
     #          access='sequential',form='formatted')
      end if
c
 100  mmb=mmb+1
      mev=nmev(mmb)
      if (mev.ge.nstep) mev=nstep
      if (mev.eq.0) go to 650
      betam=beta(mev)
      multol=500.D0*dfloat(mev+1000)*epsm
      sputol=multol
c
c---------------------------------------------------------------------
c     calculate the eigenvalues and test for the spurious values
c---------------------------------------------------------------------
c
      call compev(alpha,beta,v1,v2,vs,w,gr,multol,sputol,mp,mp2,
     #            mev,ndis)
c
      if (ndis.eq.0) go to 650
      evmax=gr(ndis)
      loop=ndis
c
c---------------------------------------------------------------------
c
      call lump(vs,v1,gr,w,reltol,sputol,scale2,mp,mp2,loop)
c
c---------------------------------------------------------------------
c
c      if (ndis.ne.loop) then
c          write(luttyo,230) ndis,loop,mev
c      end if
c
      ndis=loop
c
c---------------------------------------------------------------------
c     calculate mingaps for distinct t(1,mev) eigenvalues.
c---------------------------------------------------------------------
c
c     if igap = 0 we do not order eigenvalues by increasing gap size
c
      igap=0
      itag=1
c
      call comgap(vs,gr,gg,mp,mp2,ndis,igap,itag)
c
c     set convergence critierion
c
      ttol=epsm*evmax
      contol=cdabs(betam)*1.D-10
c
      beta(mev)=betam
c
c---------------------------------------------------------------------
c
      call isoev(vs,gr,gg,gaptol,sputol,scale1,mp,ndis,niso)
c
c---------------------------------------------------------------------
c
c     write distinct eigenvalues of T-matrix to file
      if (recflg) then
          write(ludisk,270) ndis,niso,mev,multol,sputol
          write(ludisk,280) (i,mp(i),vs(i),gg(i),mp2(i), i=1,ndis)
          write(ludisk,290) ndis, (mp(i), i=1,ndis)
      end if
c
      if (niso.eq.0) then
          write(luttyo,320)
          iconv=1
          go to 370
      end if
c
c---------------------------------------------------------------------
c     error estimates for isolated good T-eigenvalues
c---------------------------------------------------------------------
c
      it=mxinit
c
      do 330 i=1,nstep
         gr(i)=0.D0
         gc(i)=0.D0
 330  continue
c
      gr(1)=one
      gc(1)=one
c
      call inverr(alpha,beta,v1,v2,vs,epsm,gr,gc,g,gg,
     #            mp,mp2,mev,ndis,niso,it)
c
      write(luttyo,340) contol
c
      do 350 i=1,niso
 350    if (abs(g(i)).gt.contol) go to 370
c
      iconv=1
c
      write(luttyo,360) contol
c
 370  ngood=0
      do 390 i=1,ndis
      if (mp(i).eq.0) go to 390
      ngood=ngood+1
      mp(ngood)=mp(i)
      v2(ngood)=vs(i)
      w(ngood)=w(i)
      g(ngood)=g(i)
      gg(ngood)=gg(i)
 390  continue
c
c---------------------------------------------------------------------
c     sort goodev in the order of decreasing weighting factor
c---------------------------------------------------------------------
c
      write (luttyo,440) ngood,niso,ndis
c
      do 450 l=1,ngood-1
        t0=cdabs(w(l))
        do 450 m=l+1,ngood
          t1=cdabs(w(m))
          if (t1.gt.t0) then
            t0=t1
            wt=w(m)
            w(m)=w(l)
            w(l)=wt
            egv=v2(m)
            v2(m)=v2(l)
            v2(l)=egv
            gtp=g(m)
            g(m)=g(l)
            g(l)=gtp
            ggtp=gg(m)
            gg(m)=gg(l)
            gg(l)=ggtp
          end if
 450    continue
c
        ndiag=mev
c
c---------------------------------------------------------------------
c     repeat if the calculation did not converge
c---------------------------------------------------------------------
c
      beta(mev)=betam
      if (iconv.ne.1) go to 100
c
c     end of loop on different size T-matrices allowed.
c
 650  if (recflg) then
        close (unit=ludisk)
        open (unit=ludisk,file=ername,status='unknown',
     #       access='sequential',form='formatted')
        write (ludisk,700)
        write (ludisk,710) (i,v2(i),cdabs(w(i)),g(i),gg(i),i=1,ngood)
        close (unit=ludisk)
      end if
c
      write(luttyo,750) ngood,ndiag
      write(lulog,750) ngood,ndiag
c
      return
c
c=====================================================================
c     format statements
c=====================================================================
c
  80  format(' on return from CMTQLI error flag was not zero'/)
 230  format(/' after lump ndis,loop,mev=',3i6/)
 440  format(/i6,' good T-eigenvalues have been computed'/
     #       i6,' of these are isolated'/
     #       i6,'=number of distinct T-eigenvalues computed'/)
 270  format(/'ndis,niso,mev = ',3i6,/,'multol,sputol = ',g14.7,
     #       3x,g14.7,/,'evno',3x,'mult',6x,'Re(dev)',8x,'Im(dev)',
     #       10x,'mingap',10x,'index')
 280  format(i4,2x,i5,3x,g14.7,2x,g14.7,3x,g14.7,2x,i5)
 290  format(/i6,'=ndis, T-multiplicities (0 means  spurious)'/(20i3))
 320  format(' all computed good T-eigenvalues are T-multiple'/
     # ' therefore these eigenvalues are assumed to have converged')
 340  format(' convergence is tested using the convergence tolerance',
     #        e13.4)
 360  format(' all computed error estimates were less than',e15.4/
     #       ' therefore procedure terminates')
 700  format('c i',14x,'eval',17x,'|wt|',10x,'error',9x,'mingap'/'c')
 710  format(i4,3x,g12.6,2x,g12.6,3x,g12.6,3x,g12.6,3x,g12.6)
 750  format(/,2x,'# of good eigenvalues : ',i4,/,2x,'Actual ',
     #     'Lanczos steps used in the eigenvalue estimation : ',i4)
c
 800  stop
      end
