c*********************************************************************
c
c                       ================
c                       PROGRAM : EPREGV
c                       ================
c
c	This program calculates the eigenvalues and weighting factors
c       for the general complex matrix using Rutishauser Algorithm,
c       described by R. G. Gordon and T. Messenger in "Electron-Spin
c       Relaxation in Liquids", L. T. Muus and P. W. Atkins, (1972).
c
c       Includes :
c               stddim.inc
c               stdio.inc
c               eprmat.inc
c               eprdat.inc
c               timer.inc
c               fnames.inc
c               rndoff.inc
c
c       Uses :
c               getids.f
c               setnam.f
c               setflg.f
c               rdfdat.f
c               wrfdat.f
c               wrfprm.f
c               fbaso.f
c               fbasd.f
c               matrxo.f
c               matrxd.f
c               stvec.f
c               csqz1.f
c               cqrt1.f
c
c*********************************************************************
c
      program epregv
c
      implicit none
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'eprmat.inc'
      include 'fnames.inc'
      include 'timer.inc'
      include 'rndoff.inc'
c
      character*5 flgstr
      logical fexist 
c
      integer i,j,k,m,n,ncalc,ierr,jcol,kcol,ipsold
      integer bdw,bdwtmp,indx,mxit,nstpold
c
      double precision zr,zi,zero,wr,sqtol,tol,fld,dfield
c
      dimension indx(xdimg),wr(xdimg)
c
      complex*16 czero,cone,ci,cmat,x,wt,cwsp,spec,stv
      parameter (zero=0.D0,czero=(0.D0,0.D0),cone=(1.D0,0.D0),
     #           ci=(0.D0,1.D0))
      dimension cmat(xdimg,xbdwg),cwsp(xdimg),x(xdimg),stv(xdimg,5),
     #          spec(mxnpt)
c
      equivalence (wr(1),cwsp(1))
c
c#####################################################################
c
c----------------------------------------------------------------------
c Timing call changed for compatibility with UNIX
c      call cputime(cpu0,ircode)
c----------------------------------------------------------------------
      cputot = dtime( tarray )
c
      write (luttyo,1000)
      write (luttyo,1010)
c
      call getids(n,mxcalc)
c
c======================================================================
c     loop over calculations
c======================================================================
c
      do 900 ncalc=1,n
c
c---------------------------------------------------------------------
c     construct file names
c---------------------------------------------------------------------
c
      fileid = files(ncalc)
      ips=idxps(ncalc)
      call setnam
      flgstr = flags(ncalc)
      call setflg(flgstr)
c
c----------------------------------------------------------------------
c     open log file and write out header 
c----------------------------------------------------------------------
c
      open (unit=lulog,file=rlname,access='sequential',
     #      form='formatted')
c
      rewind (unit=lulog)
c
      write (luttyo,1000)
      write (lulog,1000)
      write (luttyo,5000) ncalc,prname(:namlth)
      write (lulog,5000) ncalc,prname(:namlth)
c
c----------------------------------------------------------------------
c     read in parameters from parameter file
c----------------------------------------------------------------------
c
      inquire(file=prname,exist=fexist)
c
      ipsold=ips      
      if (fexist) then
        call rdfdat(prname,ierr)
        ips=ipsold
      else
        write (luttyo,1100) prname(:namlth)
        write (lulog,1100) prname(:namlth)
        go to 999
      endif
c
      if (ierr.eq.1) then
        write (luttyo,1110) prname(:namlth)
        write (lulog,1110) prname(:namlth)
        go to 999
      else if (ierr.ne.0) then
        write (luttyo,1120) prname(:namlth)
        write (lulog,1120) prname(:namlth)
        go to 999
      end if
c
c----------------------------------------------------------------------
c     check to make sure that Exact Eigenvalues are desired
c----------------------------------------------------------------------
c
      if (itype.ne.5) then
        write (luttyo,5100) 
        write (lulog,5100)
        go to 999
      end if
c
c---------------------------------------------------------------------
c	print out parameter set
c---------------------------------------------------------------------
c
      call wrfprm(prname,namlth,lulog)
c
c---------------------------------------------------------------------
c     calculate the matrix elements
c---------------------------------------------------------------------
c
      write(luttyo,1040)
      write(lulog,1040) 
c
      if (ips.ne.0) then
        call fbaso(ixname)
        if (ndim.gt.xdimg) then
          write (luttyo,1060)
          write (lulog,1060)
          go to 999
        end if          
        call matrxo(ierr)
      else
        call fbasd(ixname)
        if (ndim.gt.xdimg) then
          write (luttyo,1060)
          write (lulog,1060)
          go to 999
        end if          
        call matrxd(ierr)
      end if
c
      if (ierr.ne.0) then
        write (luttyo,1070) 
        write (lulog,1070)
        if (ierr.eq.1) then
          write (luttyo,1080)
          write (lulog,1080) 
        end if
        go to 999
      end if
c                                     ** store matrix **
      if (stmflg) then 
        open (unit=ludisk,file=mtname,status='unknown',
     #       access='sequential',form='unformatted')
        write (luttyo,4000) mtname(:namlth)
        write (lulog,4000)  mtname(:namlth)
        rewind (unit=ludisk)
        write (ludisk) (zdiag(1,i),zdiag(2,i),i=1,ndim)
        write (ludisk) (jzmat(i),i=1,ndim+1) 
        write (ludisk) (izmat(i),zmat(i),i=1,nelim)
        write (ludisk) (kzmat(i),i=1,ndim+1)
        write (ludisk) (izmat(mxel-i+1),zmat(mxel-i+1),i=1,nelre)
        close (unit=ludisk)
      end if
c
 100  cpu1 = dtime( tarray )
      write (luttyo,1250) ndim,neltot,cpu1
      write (lulog,1250) ndim,neltot,cpu1
c
c---------------------------------------------------------------------
c     starting vector calculation
c---------------------------------------------------------------------
c
      nstpold=nstp
      call stvec(stv,x,cwsp,cmat,xdim,0)
c                              ** store starting vector **
      if ( (ips.eq.1).or.stvflg ) then
        open (unit=ludisk,file=vtname,status='unknown',
     #        access='sequential',form='unformatted')
        write (luttyo,4000) vtname(:namlth)
        write (lulog,4000)  vtname(:namlth)
        write (ludisk) (x(i),i=1,ndim)
        close (unit=ludisk)
      end if
c
      cpu2 = dtime( tarray )
c
      if (ips.eq.1) then      
          write (luttyo,2020) nelv,cpu2
          write (lulog,2020) nelv,cpu2
      end if
c
c---------------------------------------------------------------------
c     convert the matrix into banded storage format
c---------------------------------------------------------------------
c
      write (luttyo,2200)
      write (lulog,2200)
c
      do 105 j=1,xbdwg
      do 105 i=1,ndim
 105    cmat(i,j)=czero
c
      j=1
      k=1
c
      bdw=0
      do 110 m=1,ndim
        if ((abs(zdiag(1,m)).gt.rndoff).or.
     #      (abs(zdiag(2,m)).gt.rndoff)) then
          cmat(m,1)=dcmplx(zdiag(1,m),zdiag(2,m)) 
        endif
c
        if (jzmat(m).ne.jzmat(m+1)) then
          jcol=izmat(jzmat(m+1)-1)
        else
          jcol=m
        end if
c
        if (kzmat(m).ne.kzmat(m+1)) then
          kcol=izmat(mxel-kzmat(m+1)+2)
        else
          kcol=m
        end if
c
        bdwtmp=max(jcol,kcol)-m+1        
        if (bdwtmp.gt.xbdwg) then
            write (luttyo,1200)
            stop
        end if
        if (bdwtmp.gt.bdw) bdw=bdwtmp
c
        do 120 n=m+1,max(jcol,kcol)
        if ((n.le.jcol).and.(n.eq.izmat(j))) then
          zi=zmat(j)
          j=j+1
        else
          zi=zero
        endif
        if ((n.le.kcol).and.(n.eq.izmat(mxel-k+1))) then
          zr=zmat(mxel-k+1)     
          k=k+1
        else
          zr=zero
        endif
c
        if ((abs(zr).gt.rndoff).or.(abs(zi).gt.rndoff)) then
          cmat(m,n-m+1)=dcmplx(zr,zi)
        endif
c
 120    continue
c
      if ((j.ne.jzmat(m+1)).or.(k.ne.kzmat(m+1))) then
        write (luttyo,2220)
        stop
      endif
c
 110  continue
c
      if (bdw.gt.xbdwg) then
          write (luttyo,1200) bdw
          stop
      end if
      write (luttyo,2230) bdw
      write (lulog,2230) bdw
c
c---------------------------------------------------------------------
c     convert the complex symmetric matrix into tridiagonal matrix
c---------------------------------------------------------------------
c
      sqtol=zero
c
      call csqz1(cmat,x,xdimg,xbdwg,ndim,bdw,sqtol)
c
 200  cpu3 = dtime( tarray )
      write (luttyo,2250) cpu3
      write (lulog,2250) cpu3
c                             ** store tridiagonal matrix **
c
      if (tdlflg) then
        write (luttyo,4000) tdname(:namlth)
        write (lulog,4000) tdname(:namlth)
c
        open (unit=ludisk,file=tdname,status='unknown',
     #       access='sequential',form='unformatted')
        rewind (unit=ludisk)
        write(ludisk) (cmat(j,1),cmat(j,2),j=1,ndim)
        close (unit=ludisk)
      end if
c
c---------------------------------------------------------------------
c     Solve tridiagonal matrix for eigenvalues and eigenvectors
c---------------------------------------------------------------------
c
      write (luttyo,2300)
      write (lulog,2300)
c
      mxit=100
c
      do 150 i=1,ndim
 150    cwsp(i)=czero
c
      call cqrt1(cmat(1,1),cmat(1,2),x,ndim,mxit,tol,cwsp,ierr)
c
      cpu4 = dtime( tarray )
      write (luttyo,2250) cpu4
      write (lulog,2250) cpu4
c
      if (ierr.ne.0) then
        write (luttyo,2340)
        call exit
      end if
c
      do 300 i=1,ndim
        x(i)=x(i)*x(i)
        wr(i)=cdabs(x(i))
 300  continue
c
      call indexx(ndim,wr,indx)
c
c---------------------------------------------------------------------
c     write the eigenvalues into formatted file
c---------------------------------------------------------------------
c
      open (unit=ludisk,file=egname,status='unknown',
     #     access='sequential',form='formatted')
      write (ludisk,2420) (i,indx(i),cmat(indx(i),1),
     #     x(indx(i)),i=1,ndim)
      close (unit=ludisk)
c
      ngood=ndim
c
c---------------------------------------------------------------------
c     calculate cw spectrum for the off-diagonal space
c---------------------------------------------------------------------
c
      if (ips.ne.1) go to 999
c
      write (luttyo,2270)
      write (lulog,2270)
c
      nev=0
 305  nev=nev+1
      if (wr(indx(nev)).ge.wtol) go to 305
      write (luttyo,2280) nev,wtol
      write (lulog,2280) nev,wtol
c
      dfield=(fieldf-fieldi)/dble(nfield-1)
c
      do 310 i=1,mxnpt
 310    spec(i)=czero
c
      do 320 i=1,nfield
        fld=fieldi+dfloat(i-1)*dfield
        do 330 j=1,nev
          k=indx(j)
          spec(i)=spec(i)+x(k)/(cmat(k,1)+shiftr+(b0+shifti+fld)*ci)
 330  continue
 320  continue
c
      open (unit=ludisk,file=spname,status='unknown',
     #     access='sequential',form='formatted')
c    frequency sweep
c      write (ludisk,3200) (b0+fieldi+dfloat(i-1)*dfield,
c     #                     spec(i),i=1,nfield)
c    field sweep
      write (ludisk,3200) (b0+fieldi+dfloat(i-1)*dfield,
     #                     spec(nfield-i+1),i=1,nfield)

      close (unit=ludisk)
c
 999  continue
c
c---------------------------------------------------------------------
c     update parameter file
c---------------------------------------------------------------------
c
      write (luttyo,4000) prname(:namlth)
      write (lulog,4000) prname(:namlth)
c
      if (nstpold.ne.nstp) nstp=nstpold
      call wrfdat(prname)
c
c---------------------------------------------------------------------
c     print out trailer and close log file
c---------------------------------------------------------------------
c
      write (luttyo,1000)
      write (lulog,1000)
c
      cputot = cputot+cpu1+cpu2+cpu3+cpu4
c
      write (lulog,6000) cputot
      write (luttyo,6000) cputot
      close (unit=lulog)
c
c======================================================================
c     end of loop over calculations
c======================================================================
c
 900  continue
c
c---------------------------------------------------------------------
c     exit program
c---------------------------------------------------------------------
c
      stop
c
c=====================================================================
c     format statements
c=====================================================================
c
 1000 format(//,2x,70('#'),//)
 1010 format(25x,'PROGRAM : EPREGV',/,25x,'---------------',/)
c
 1020 format(/,2x,'Eigenvectors for pS=1 space required for ',
     #       'starting vector calculation.')
 1025 format(/,2x,'Proceed without starting vector? (1/0)')
 1040 format(/,15x,'*** Calculating matrix elements ***')
 1060 format(/,20x,'*** MATRIX DIMENSION TOO LARGE ***',/,
     #     4x,'Use program compiled with bigger dimension or reduce',
     #     ' the dimension.')
 1070 format(//,10x,
     #     '*** SERIOUS ERROR ENCOUNTERED IN MATRIX GENERATION ***')
 1080 format(/,20x,'*** TOO MANY OFF-DIAGONAL MATRIX ELEMENTS ***')
 1100 format(/,20x,'*** CANNOT FIND FILE ',a,' ***',/)
 1110 format(/,20X,'*** FILE ',a,' NOT EPRLF ***',/)
 1120 format(/,20X,'*** ERROR READING FILE ',a,' ***',/)
 1200 format(/,20X,'*** BANDWIDTH : ',i4,' IS TOO WIDE!! ***',/)
 1250 format(/,2x,'Dimension of the matrix : ',i5,
     #       /,2x,'Total number of non-zero off-diagonal entries : ',i6,
     #       /,2x,'CPU time (seconds) : ',f10.2,/)
c
 2020 format(/,2x,'Number of non-zero elements in ',
     #     'starting vector : ',i5,
     #     /,2x,'CPU time (seconds) : ',f10.2)
c
 2200 format(/,15x,'*** Calculating tridiagonal matrix ***')
 2220 format(//,15x,'*** OOPS!, MATRIX FILE CORRUPTED ***',//)
 2230 format(/,2x,'BandWidth = ',i4)
 2250 format(/,2x,'CPU time (seconds) : ',f10.2)
 2300 format(/,15x,'*** Diagonalizing tridiagonal matrix ***')
 2340 format(/,5x,'!!! CQRT has not converged in',i4,' iterations !!!')
 2270 format(/,15x,'*** Calculating cw spectrum ***')
 2280 format(/,2x,i4,' eigenvalues included with wtol = ',g10.3)
 2420 format(i4,2x,i4,4x,g14.7,',',g14.7,2x,g14.7,',',g14.7)
c
 3200 format(1x,3(g14.7,','))
 4000 format(/,15x,'*** Writing ',a,' ***')
 5000 format(15x,38('='),/,
     #     15x,'Calculation no. ',I2,' using file ',a,/,
     #     15x,38('='),/)
 5100 format(//,10x,
     #     '*** NOT EXACT EIGENVALUES CALCULATION ***')
c
 6000 format(/,2x,'Total CPU time (seconds) : ',f10.2)
c
c=====================================================================
c
      end
