c*********************************************************************
c
c                       ================
c                       PROGRAM : EPRLCG
c                       ================
c
c       EPR (L)anczos (C)onjugate (G)radient
C
c       This program calculates the eigenvalues and eigenvectors.
c
c       Eigenvalues are obtained by the Lanczos routine, with all the
c       tests for spurious and redundent eigenvalues implemented.
c
c       Eigenvectors are obtained by the Conjugate Gradient routine
c       only for the good eigenvalues.
c
c	modified from EPRLF version 1.2 by Sanghyuk Lee
c
c       Includes :
c               stddim.inc
c               stdio.inc
c               eprmat.inc
c               eprdat.inc
c               timer.inc
c               fnames.inc
c               pulsep.inc
c
c       Uses :
c               getids.f
c               setnam.f
c               setflg.f
c               rdfdat.f
c               wrfdat.f
c               wrfprm.f
c               fbaso.f
c               fbasd.f
c               matrxo.f
c               matrxd.f
c               stvec.f
c               cslnlc.f
c               csval.f
c               cscglc.f
c               znormu.f
c
c*********************************************************************
c
      program eprlcg
c
      implicit none
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'eprmat.inc'
      include 'fnames.inc'
      include 'timer.inc'
      include 'pulsep.inc'
c
      character*5 flgstr
      logical fexist,pcfg
c
      complex*16 x,y,alpha,beta,czero,ci,evec,cwsp,stv
      dimension x(mxdim),y(mxdim),alpha(mxstep),beta(mxstep),
     #          evec(mxdim,mxnev),cwsp(mxdim),stv(mxdim,5)

      integer i,j,k,n,ncalc,ierr,ndone,ipsold,ival,mxstp,nstpold
c   
      double precision error,errorv,eps,wta
      parameter (czero=(0.0D0,0.0D0),ci=(0.0D0,1.0D0))
c
      complex*16 w,temp1,temp2
      dimension w(mxstep),temp1(mxstep),temp2(mxstep)
c
      complex*16 eval,v1,shift
      dimension v1(mxstep)
      complex*16 znormu,cnorm
      external znormu
c
      equivalence (y(1),cwsp(1))
c
c#####################################################################
c
c----------------------------------------------------------------------
c Timing call changed for compatibility with UNIX
c      call cputime(cpu0,ircode)
c----------------------------------------------------------------------
       cputot = dtime( tarray )
c
      write (luttyo,1000)
      write (luttyo,1010)
c
      call getids(n,mxcalc)
c
c======================================================================
c     loop over calculations
c======================================================================
c
      do 900 ncalc=1,n
c
c---------------------------------------------------------------------
c     construct file names
c---------------------------------------------------------------------
c
      fileid = files(ncalc)
      ips=idxps(ncalc)
      call setnam
      flgstr = flags(ncalc)
      call setflg(flgstr)
c
c----------------------------------------------------------------------
c     open log file and write out header 
c----------------------------------------------------------------------
c
      open (unit=lulog,file=rlname,access='sequential',
     #      form='formatted')
c
      rewind (unit=lulog)
c
      write (luttyo,1000)
      write (lulog,1000)
      write (luttyo,5000) ncalc,prname(:namlth)
      write (lulog,5000) ncalc,prname(:namlth)
c
c----------------------------------------------------------------------
c     read in parameters from parameter file
c----------------------------------------------------------------------
c
      inquire(file=prname,exist=fexist)
c
      ipsold=ips
      if (fexist) then
        call rdfdat(prname,ierr)
        ips=ipsold
      else
        write (luttyo,1100) prname(:namlth)
        write (lulog,1100) prname(:namlth)
        go to 999
      endif
c
      if (ierr.eq.1) then
        write (luttyo,1110) prname(:namlth)
        write (lulog,1110) prname(:namlth)
        go to 999
      else if (ierr.ne.0) then
        write (luttyo,1120) prname(:namlth)
        write (lulog,1120) prname(:namlth)
        go to 999
      end if
c
c----------------------------------------------------------------------
c     check to make sure that LCG calculation wanted
c----------------------------------------------------------------------
c
      if (itype.ne.3) then
        write (luttyo,5100) 
        write (lulog,5100)
        go to 999
      end if
c
      if (nort.gt.1) then
        write (luttyo,5200) 
        write (lulog,5200)
        go to 999
      end if
c
c---------------------------------------------------------------------
c	print out parameter set
c---------------------------------------------------------------------
c
      call wrfprm(prname,namlth,lulog)
c
c---------------------------------------------------------------------
c     calculate the matrix elements
c---------------------------------------------------------------------
c
      write(luttyo,1040)
      write(lulog,1040) 
c
      if (ips.ne.0) then
        call fbaso(ixname)
        if (ndim.gt.mxdim) then
          write (luttyo,1060)
          write (lulog,1060)
          go to 999
        end if          
        call matrxo(ierr)
      else
        call fbasd(ixname)
        if (ndim.gt.mxdim) then
          write (luttyo,1060)
          write (lulog,1060)
          go to 999
        end if          
        call matrxd(ierr)
      end if
c
      if (ierr.ne.0) then
        write (luttyo,1070) 
        write (lulog,1070)
        if (ierr.eq.1) then
          write (luttyo,1080)
          write (lulog,1080) 
        else if (ierr.eq.2) then
          write (luttyo,1090)
          write (lulog,1090)
        end if
        go to 999
      end if
c                                     ** store matrix **
      if (stmflg) then 
        open (unit=ludisk,file=mtname,status='unknown',
     #       access='sequential',form='unformatted')
        write (luttyo,4000) mtname(:namlth)
        write (lulog,4000)  mtname(:namlth)
        rewind (unit=ludisk)
        write (ludisk) (zdiag(1,i),zdiag(2,i),i=1,ndim)
        write (ludisk) (jzmat(i),i=1,ndim+1) 
        write (ludisk) (izmat(i),zmat(i),i=1,nelim)
        write (ludisk) (kzmat(i),i=1,ndim+1)
        write (ludisk) (izmat(mxel-i+1),zmat(mxel-i+1),i=1,nelre)
        close (unit=ludisk)
      end if
c
 100  cpu1 = dtime( tarray )
      write (luttyo,1250) ndim,neltot,cpu1
      write (lulog,1250) ndim,neltot,cpu1
c
c---------------------------------------------------------------------
c     starting vector calculation
c---------------------------------------------------------------------
c
      nstpold=nstp
      call stvec(stv,x,cwsp,evec,mxdim,0)
c
c                              ** store starting vector **
      if ( (ips.eq.1).or.stvflg ) then
        open (unit=ludisk,file=vtname,status='unknown',
     #        access='sequential',form='unformatted')
        write (luttyo,4000) vtname(:namlth)
        write (lulog,4000)  vtname(:namlth)
        write (ludisk) (x(i),i=1,ndim)
        close (unit=ludisk)
      end if
c
      cpu2 = dtime( tarray )
c
      if (ips.ne.0) then
        write (luttyo,2020) nelv,cpu2
        write (lulog,2020) nelv,cpu2
      else
        write (luttyo,2030) cpu2
        write (lulog,2030) cpu2
      end if
c
c---------------------------------------------------------------------
c     call lanczos routine
c---------------------------------------------------------------------
c
      write (luttyo,3000) 
      write (lulog,3000)
c
      do 25 i=1,mxdim
 25   y(i)=czero
c
      call cslnlc(x,y,nstep,ndim,alpha,beta)
      cpu3 = dtime( tarray )
c
      write (luttyo,3010) cpu3
      write (lulog,3010) cpu3
c                             ** store tridiagonal matrix **
      if (tdlflg) then
        write (luttyo,4000) tdname(:namlth)
        write (lulog,4000) tdname(:namlth)
        open (unit=ludisk,file=tdname,
     #       access='sequential',form='unformatted')
        rewind (unit=ludisk)
        write(ludisk) (alpha(j),beta(j),j=1,nstep)
        close (unit=ludisk)
      end if
c
c---------------------------------------------------------------------
c     call eigenvalue calculation routine
c---------------------------------------------------------------------
c
      if (evalflg) then
c				**  read eigenvalues and wt factors **
        inquire(file=egname,exist=fexist)
        if (fexist) then
          write (luttyo,4100)
          write (lulog,4100)
          open (unit=ludisk,file=egname,status='old',
     #          access='sequential',form='formatted')
          read (ludisk,4150) (j,k,temp2(i),w(i),i=1,ngood)
          close (unit=ludisk)
          go to 200
        end if
      end if
c
      write(luttyo,4200)
      write(lulog,4200)
c
      call csval(alpha,beta,temp1,temp2,v1,w)
c
      cpu4 = dtime( tarray )
      write (luttyo,3010) cpu4
      write (lulog,3010) cpu4
c
c           ** store good eigenvalues in formatted file **
c
      open (unit=ludisk,file=egname,
     #     access='sequential',form='formatted')
      write (ludisk,4220) (i,0,temp2(i),w(i),i=1,ngood)
      close (unit=ludisk)
c
 200  if (.not.evecflg) go to 999
c
c---------------------------------------------------------------------
c    open the file for storing the eigenvectors
c---------------------------------------------------------------------
c
      open (unit=ludisk,file=evname(:namlth),status='unknown',
     #     access='sequential',form='unformatted')
c
c----------------------------------------------------------------------
c     open the file for storing the error information
c----------------------------------------------------------------------
c
      if (recflg) open (unit=ludisk1,file=ername,status='unknown',
     #     access='sequential',form='formatted')
c
c---------------------------------------------------------------------
c     calculate the eigenvectors for nev important eigenvalues
c---------------------------------------------------------------------
c
      write(luttyo,4300)
      write(lulog,4300)
c
      nev=0
c
      pcfg=ipt.eq.0
c
      do 500 ival=1,ngood
      wta=cdabs(w(ival))
      if (wta.lt.wtol) go to 600
      mxstp=mxcg
      nev=nev+1
      eval=temp2(ival)
c
      if (recflg) write(ludisk1,4310) ival,eval
c
      shift=dcmplx(shiftr,shifti)
      shift=shift-eval
c
      do 520 i=1,mxdim
 520    y(i)=czero
c
      call cscglc(x,ndim,mxstp,cgtol,shift,y,ndone,error)
c
c     normalize the eigenvector
c
      cnorm=znormu(y,ndim)
      do 530 i=1,ndim
 530  y(i)=y(i)/cnorm
c
c---------------------------------------------------------------------
c     write the eigenvector into the disk
c----------------------------------------------------------------------
c
      if (ips.ne.0) then
        write (ludisk) (y(i),i=1,ndim)
      else
c          * convert the diagonal eigenvectors into off-diagonal
c                 using the pulse propagator *
        k=1
        do 360 j=1,ndimo
          if (pid(j).eq.1) then
            y(j)=-pp(k)*y(k)
            k=k+1
          else
            y(j)=-pp(k)*y(k)-pp(k+1)*y(k+1)
            k=k+2
          end if
 360    continue
        write (ludisk) (y(i),i=1,ndimo)
      end if
c
 500  continue
c
 600  close (unit=ludisk)
c
      if (recflg) close (unit=ludisk1)
c
      write (luttyo,4330) nev,ngood
      write (lulog,4330) nev,ngood
c
      cpu5 = dtime( tarray )
c
      write (luttyo,3010) cpu5
      write (lulog,3010) cpu5
c
 999  continue
c
c---------------------------------------------------------------------
c      update parameter file
c---------------------------------------------------------------------
c
      write (luttyo,4000) prname(:namlth)
      write (lulog,4000) prname(:namlth)
c
      if (ips.eq.1) nevo=nev
      if (nstp.ne.nstpold) nstp=nstpold
      call wrfdat(prname)
c
c---------------------------------------------------------------------
c     print out trailer and close log file
c---------------------------------------------------------------------
c
      write (luttyo,1000)
      write (lulog,1000)
c
      cputot = cputot+cpu1+cpu2+cpu3+cpu4+cpu5
c
      write (lulog,6000) cputot
      write (luttyo,6000) cputot
      close (unit=lulog)
c
c======================================================================
c     end of loop over calculations
c======================================================================
c
 900  continue
c
c---------------------------------------------------------------------
c     exit program
c---------------------------------------------------------------------
c
      stop
c
c=====================================================================
c     format statements
c=====================================================================
c
 1000 format(//,2x,70('#'),//)
 1010 format(25x,'PROGRAM : EPRLCG',/,25x,'---------------',//)
c
 1040 format(/,15x,'*** Calculating matrix elements ***')
 1060 format(/,20x,'*** MATRIX DIMENSION TOO LARGE ***',/,
     #     4x,'Use program compiled with bigger dimension or reduce',
     #     ' the dimension.')
 1070 format(//,10x,
     #     '*** SERIOUS ERROR ENCOUNTERED IN MATRIX GENERATION ***')
 1080 format(/,20x,'*** TOO MANY OFF-DIAGONAL MATRIX ELEMENTS ***')
 1090 format(/,20x,'*** MATRIX DIMENSION TOO LARGE ***')
 1100 format(/,20x,'*** CANNOT FIND FILE ',a,' ***',/)
 1110 format(/,20X,'*** FILE ',a,' NOT EPRLF ***',/)
 1120 format(/,20X,'*** ERROR READING FILE ',a,' ***',/)
 1250 format(/,2x,'Dimension of the matrix : ',i5,
     #       /,2x,'Total number of non-zero off-diagonal entries : ',i6,
     #       /,2x,'CPU time (seconds) : ',f10.2,/)
c
 2020 format(/,2x,'Number of non-zero elements in ',
     #     'starting vector : ',i5,
     #     /,2x,'CPU time (seconds) : ',f10.2)
 2030 format(/,2x,'CPU time for t1-dep. starting vector (seconds)',
     #     ' : ',f10.2)
c
 3000 format(/,15x,'*** Calculating tridiagonal matrix ***')
 3010 format(/,2x,'CPU time (seconds) : ',f10.2)
c
 4000 format(/,20x,'*** Writing ',a,' ***')
 4100 format(/,15x,'*** Reading eigenvalues ***')
 4150 format(i4,2x,i4,4x,g14.7,1x,g14.7,2x,g14.7,1x,g14.7)
 4200 format(/,15x,'*** Calculating eigenvalues ***')
 4210 format(/,2x,'# of good eigenvalues : ',i4,/,2x,'Actual ',
     #     'Lanczos steps used in the eigenvalue estimation : ',i4)
 4220 format(i4,2x,i4,4x,g14.7,',',g14.7,2x,g14.7,',',g14.7)
 4300 format(/,15x,'*** Calculating eigenvectors ***')
 4310 format(/'** Eigenvalue ',i4,'  =  ( ',g14.7,' , ',g14.7,')'/)
 4330 format(/,2x,i4,' eigenvectors calculated out of ',i4,
     #     ' good eigenvalues')
 5000 format(15x,38('='),/,
     #     15x,'Calculation no. ',I2,' using file ',a,/,
     #     15x,38('='),/)
 5100 format(//,10x,
     #     '*** LANCZOS+CG CALCULATION NOT REQUESTED ***')
 5200 format(/,10x,
     #     '*** USE PROGRAM MOMD FOR ORIENTATIONAL AVERAGE***')
c
 6000 format(/,2x,'Total CPU time (seconds) : ',f10.2)
c
c=====================================================================
c
      end
