c**********************************************************************
c       
c                       ==============
c                       PROGRAM : LBLF
c                       ==============
c
c       This program will prompt the user to enter the parameters 
c       required as input to the EPRLF spectral calculation program.
c
c
c       rewritten by DEB 13-MAR-92 from program LBLF by DJS and JPG
c       Uses new filename convention, subroutine GETIDS to input file i.d.
c       Also uses new routine PRMPT, and functions FTOKEN and ITOKEN
c       to input parameter values. This permits the user simply to type
c       <ENTER> when it is not desired to change a given parameters.
c
c       modified by SHL
c
c       Includes:
c               maxl.inc
c               stddim.inc
c               stdio.inc
c               rndoff.inc
c               eprdat.inc
c               fnames.inc
c               indexf.inc
c       Uses:
c               getids.f
c               setnam.f
c               prmpt.f
c               rdfdat.f
c               wrfdat.f
c               fbaso.f
c               fbasd.f
c               wrfbas.f
c               wrfprm.f
c               cd2km.f
c               ipar.f
c
c**********************************************************************
c
      program lblf
c
      implicit none
c
      include 'stddim.inc'
      include 'maxl.inc'
      include 'stdio.inc'
      include 'physcn.inc'
      include 'rndoff.inc'
      include 'eprdat.inc'
      include 'fnames.inc'
      include 'indexf.inc'
c
      logical fprmpt,fexist,flagav,axiala,axialg
      integer i,ibasis,ierr,inlemx,inlomx,inkmx,inmmx,inpnmx,
     #        j,k,l,nfiles,ldel,kdel
c
      double precision d2km(2,5,5),tmp
      character*80 line
c
      double precision zero,half,dsq23,one
      parameter (zero=0.0D0,half=0.5D0,one=1.0D0)
c
      integer ipar,itoken
      double precision ftoken
      logical prmpt
      external ipar,prmpt,itoken,ftoken
c
c######################################################################
c
      dsq23=dsqrt(2.0D0/3.0D0)
      flagav=.false.
c
      write (luttyo,1000)
      write (luttyo,1010) 
c
      call getids(nfiles,1)
      if (nfiles.eq.0) stop
      fileid=files(1)
      ips=idxps(1)
      call setnam
c
c----------------------------------------------------------------------
c     check for existence and open parameter file
c----------------------------------------------------------------------
c
      inquire(file=prname,exist=fexist)
      if (fexist) then
        write (luttyo,1040) prname(:namlth)
        call rdfdat(prname,ierr)
        if (ierr.ne.0) call wrfprm(prname,namlth,luttyo)
        if (ips.ne.idxps(1)) then
           write (luttyo,1041)
           ips=idxps(1)
        end if
      end if
c
c----------------------------------------------------------------------
c     initialize spherical tensors
c----------------------------------------------------------------------
        do 10 j=1,5
          faa(j)=zero
          fgm(j)=zero
          do 9 i=1,2
            fam(i,j)=zero
            fgd(i,j)=zero
            fad(i,j)=zero
    9       continue
 10       continue
c
c----------------------------------------------------------------------
c     get g-tensor
c----------------------------------------------------------------------
c
 11   write (luttyo,2000) gxx,gyy,gzz
      fprmpt=prmpt(line)
      if (fprmpt) then
        gxx=ftoken( line )
        gyy=ftoken( line )
        gzz=ftoken( line )
      end if

      if ( abs(gxx) .lt. rndoff  .or.
     #     abs(gyy) .lt. rndoff  .or.
     #     abs(gzz) .lt. rndoff) go to 11
c
c----------------------------------------------------------------------
c     get nuclear spin and A-tensor (in gauss)
c----------------------------------------------------------------------
c
      write (luttyo,2010) in2
      fprmpt=prmpt(line)
      if (fprmpt) in2=itoken( line )
      if (in2.le.0) then
        in2=0
      else
        write (luttyo,2020) axx,ayy,azz
        fprmpt=prmpt(line)
        if (fprmpt) then
          axx=ftoken(line)
          ayy=ftoken(line)
          azz=ftoken(line)
        end if
c
        write (luttyo,2021) gamman
        fprmpt=prmpt(line)
        if (fprmpt) gamman=ftoken(line)
c
      end if
c
c----------------------------------------------------------------------
c     get center field (in gauss)
c----------------------------------------------------------------------
c
 20   write (luttyo,2030) b0
      fprmpt=prmpt(line)
      if (fprmpt) b0=ftoken(line)
c
      if (b0.lt.zero) b0 = abs(b0)
      if (b0.lt.rndoff) go to 20
c
      if ( (in2.le.0) .or.
     #    (dabs(axx) .lt. rndoff .and. 
     #     dabs(ayy) .lt. rndoff .and.
     #     dabs(azz) .lt. rndoff)     ) then
        in2 = 0
        axx = zero
        ayy = zero
        azz = zero
        zeen = zero
      endif

c----------------------------------------------------------------------
c     Calculate spherical components of tensors in magnetic frames
c----------------------------------------------------------------------
c                                        *** Electronic Zeeman ***
      g0=(gxx+gyy+gzz)/3.0D0
      fgm(1)=half*(gxx-gyy)*(b0/g0)
      fgm(3)=dsq23*(gzz-half*(gxx+gyy))*(b0/g0)
      fgm(5)=fgm(1)
      axialg = dabs(fgm(1)).lt.rndoff
c                                        *** Hyperfine ***
      a0=(axx+ayy+azz)/3.0D0
      faa(1)=half*(axx-ayy)
      faa(3)=dsq23*(azz-half*(axx+ayy))
      faa(5)=faa(1)
      axiala = dabs(faa(1)).lt.rndoff
c                                        *** Nuclear Zeeman ***
      zeen = gamman*b0*hbar/(g0*betae)
      if(abs(zeen).le.rndoff) zeen=zero
c
c----------------------------------------------------------------------
c     get diffusion model index
c----------------------------------------------------------------------
c
      write (luttyo,2040) ipdf
      fprmpt=prmpt(line)
      if (fprmpt) ipdf=itoken(i)
      if (ipdf.gt.2) ipdf=2
      if (ipdf.lt.0) ipdf=0
c
c----------------------------------------------------------------------
c     get diffusion tensor (per second)
c----------------------------------------------------------------------
c
      write (luttyo,2050) dx,dy,dz
      fprmpt=prmpt(line)
      if (fprmpt) then
        dx=ftoken(line)
        dy=ftoken(line)
        dz=ftoken(line)
      end if
c
c     set criterion for weighting factor to store eigenvector
c
      tmp=9.99D7
      do 30 i=1,6
        if ((dx.gt.tmp).and.(dy.gt.tmp).and.(dz.gt.tmp)) then
          go to 40
        else
          tmp=tmp*1.0D-1
        end if
 30   continue
c
 40   if (i.eq.1) then
        wtol=5.0D-2
      else if (i.eq.2) then
        wtol=1.0D-2
      else if (i.eq.3) then
        wtol=2.0D-3
      else if (i.eq.4) then
        wtol=8.0D-4
      else if (i.eq.5) then
        wtol=4.0D-4
      else if (i.eq.6) then
        wtol=1.0D-4
      else
        wtol=1.0D-5
      end if
c
      write (luttyo,2055) wtol
      fprmpt=prmpt(line)
      if (fprmpt) wtol=ftoken(line)
c
c----------------------------------------------------------------------
c     get more motional parameters if needed
c----------------------------------------------------------------------
c
      if (ipdf.eq.0) go to 200
c
      if (ipdf.eq.2) then
        write (luttyo,2060) tl,tkxy,tkzz
      else
        write (luttyo,2070)  tl,tkxy,tkzz
      end if
      fprmpt=prmpt(line)
      if (fprmpt) then
        tl=ftoken(line)
        tkxy=ftoken(line)
        tkzz=ftoken(line)
      endif
      if ((ipdf.eq.2).and.(abs(tl).lt.rndoff).and.
     #     (abs(tkxy).lt.rndoff).and.(abs(tkzz).lt.rndoff)) then
        flagav=.true.
        go to 200
      else
        flagav=.false.
        djfprp=zero 
      end if
c
c----------------------------------------------------------------------
c     get non-brownian model parameters
c----------------------------------------------------------------------
c
      write (luttyo,2080) pl,pkxy,pkzz
      fprmpt=prmpt(line)
c
      if (fprmpt) then
          pl=ftoken(line) 
          pkxy=ftoken(line)
          pkzz=ftoken(line)
      end if
c
 200  continue
c
c----------------------------------------------------------------------
c     get number of sites and jump rate for site jump model
c----------------------------------------------------------------------
c
      write (luttyo,2090) ist,djf
      fprmpt=prmpt(line)
      if (fprmpt) then
        ist=itoken(line)
        djf=ftoken(line)
      end if
c
c----------------------------------------------------------------------
c     get perpendicular rate for anisotropic viscosity term
c----------------------------------------------------------------------
c
      if (flagav.and.(ist.eq.0)) then
        write (luttyo,2073) djfprp
        fprmpt=prmpt(line)
        if (fprmpt) djfprp=ftoken(line)
      else
        djfprp=zero
      end if            
c
c----------------------------------------------------------------------
c     get Heisenberg spin exchange rate (should be positive)
c----------------------------------------------------------------------
c
      write(luttyo,2100) oss
      fprmpt=prmpt(line)
      if (fprmpt) oss=abs(ftoken(line))
      if (oss.lt.rndoff) oss=zero
c
c----------------------------------------------------------------------
c     get the number of orientations for MOMD calculation
c----------------------------------------------------------------------
c
      write (luttyo,2105) nort
      fprmpt=prmpt(line)
      if (fprmpt) nort=itoken(line)
      if (nort.lt.0) nort=0
c
c      if ((abs(zeen).gt.rndoff).and.(nort.gt.1)) then
c        write (luttyo,2107)
c        zeen=zero
c        gamman=zero
c      end if
c
c----------------------------------------------------------------------
c     get potential parameters
c----------------------------------------------------------------------
c
      write (luttyo,2110) ipt
      fprmpt=prmpt(line)
      if (fprmpt) ipt=itoken(line)
      if (ipt.lt.0) ipt=0
      if (ipt.gt.6) ipt=6
      if (ipt.gt.0) then
        write(luttyo,2120)
        do 220 j=1,ipt
          write(luttyo,2130) j,(ixp(j,i),i=1,2),cxp(j)
          fprmpt=prmpt(line)
          if (fprmpt) then
            ixp(j,1)=itoken(line)
            ixp(j,2)=itoken(line)
            cxp(j)=ftoken(line)
          end if
 220    continue
c
        do 230 j=1,ipt
          i=ixp(j,1)
          if(i.gt.4) i=4
          if(i.lt.2) i=2
          if(i.eq.3) i=2
          ixp(j,1)=i
          i=ixp(j,2)
          if(i.gt.ixp(j,1)) i=ixp(j,1)
          if(i.lt.0) i=0
          if(ipar(i).eq.-1) i=i-1
          ixp(j,2)=i
 230    continue
      end if
c
      do 240 j=1,5
        do 245 i=1,5
          cpot(i,j)=zero
 245    continue
 240  continue
c
      lptmx=0
      kptmx=0
c
      do 250 i=1,ipt
        l=ixp(i,1)
        k=ixp(i,2)
        if(l.gt.lptmx) lptmx=l
        if(k.gt.kptmx) kptmx=k
        l=1+l/2
        k=1+k/2
        cpot(l,k)=cxp(i)
 250  continue
c
      if (lptmx.ge.2) then
        lband=lptmx*2
      else
        lband=2
      end if
c
      kband=kptmx*2
      if (abs(dx-dy).gt.rndoff)  kband=kband+2
c
c----------------------------------------------------------------------
c Check specified motional model parameters for consistency
c----------------------------------------------------------------------
c
c     *** Non-Brownian motion: must have at least one nonzero residence
c         time, and no potential specified
c
      if (ipdf .eq. 1) then
         tl = abs(tl)
         tkxy = abs(tkxy)
         tkzz = abs(tkzz)
c
         if ((tl .lt. rndoff) .and. (tkxy .lt. rndoff) .and.
     #      (tkzz .lt. rndoff) )  then
            write (luttyo,5004)
            ipdf=0
         endif
c
         if (ipt .gt. 0) then
            write (luttyo,5005)
            ipdf=0
         endif
      endif
c
c     *** Anisotropic viscosity model: must have potential and
c         no discrete jump motion specified
c
      if (ipdf .eq. 2) then
        if (ipt .eq. 0) then
          write (luttyo,5006)
          ipdf=0
          flagav=.false.
        endif
c
        if (ist .gt. 0) then
          write (luttyo,5007)
          ipdf=0
          flagav=.false.
        endif
      endif
c
c     *** Brownian motion model: set residence times to zero
c
      if (ipdf .eq. 0) then
         tl   = zero
         tkxy = zero
         tkzz = zero
      endif
c
c----------------------------------------------------------------------
c     get director tilt if there is a non-vanishing potential
c----------------------------------------------------------------------
c
      if ((ipt.gt.0).and.(nort.le.1)) then
        write(luttyo,2140) psi
        fprmpt=prmpt(line)
        if (fprmpt) psi=ftoken(line)
      else
        psi=zero
      end if
c
      ipsi0=0
      if (nort.gt.1) ipsi0=1
      if((abs(psi).gt.rndoff).and.
     #      (abs(psi-180.0d0).gt.rndoff)) ipsi0=1
c
c----------------------------------------------------------------------
c     Get diffusion tilt angles
c----------------------------------------------------------------------
c
      write(luttyo,2160) ald,bed,gad
      fprmpt=prmpt(line)
      if (fprmpt) then
        ald=ftoken(line)
        bed=ftoken(line)
        gad=ftoken(line)
      end if
c
      bed = dabs( bed )
      itd = 0
      if (dabs(ald).gt.rndoff .or. dabs(gad).gt.rndoff
     #    .or. bed.gt.rndoff) itd=1
c
c----------------------------------------------------------------------
c     Get magnetic tilt angles
c----------------------------------------------------------------------
      write(luttyo,2162) alm,bem,gam
      fprmpt=prmpt(line)
      if (fprmpt) then
         alm=ftoken(line)
         bem=ftoken(line)
         gam=ftoken(line)
      end if
c
      bem = dabs( bem )
      if (axialg .and. dabs(alm).gt.rndoff) then
        alm=zero
        write(luttyo,5014)
      end if
c
c     *** Only beta tilt angles allowed if all tensors are axial
c
      if (axialg.and.axiala.and.(dabs(alm).gt.rndoff.or.
     #    dabs(gam).gt.rndoff.or.dabs(gad).gt.rndoff) ) then
        alm=zero
        gam=zero
        gad=zero
        write(luttyo,5015)
      end if
c
      itm=0
      if (dabs(alm).gt.rndoff .or. bem.gt.rndoff 
     #    .or. dabs(gam).gt.rndoff)     itm=1
c
c----------------------------------------------------------------------
c     Get relaxation parameters
c----------------------------------------------------------------------
c
      write(luttyo,2165) t2edi,t2ndi,t2efi
      fprmpt=prmpt(line)
      if (fprmpt) then
         t2edi=ftoken(line)
         t2ndi=ftoken(line)
         t2efi=ftoken(line)
      end if
c
      write(luttyo,2166) t1edi,t1ndi
      fprmpt=prmpt(line)
      if (fprmpt) then
         t1edi=ftoken(line)
         t1ndi=ftoken(line)
      end if
c
c----------------------------------------------------------------------
c     Set A tensor in magnetic (g-tensor) frame
c----------------------------------------------------------------------
      if (itm .eq. 0) then
c                           *** no tilt: copy tensor directly
        do 252 j=1,5
            fam(1,j)=faa(j)
  252       continue
      else
c                          *** transform A tensor into g axis system
c
       call cd2km(d2km,alm,bem,gam)
       do 255 i=1,5
         do 254 j=1,5
           fam(1,i)=fam(1,i)+d2km(1,i,j)*faa(j)
           fam(2,i)=fam(2,i)+d2km(2,i,j)*faa(j)
  254      continue
  255    continue
      end if
c
c----------------------------------------------------------------------
c     Set F_g and F_A tensors in the diffusion frame
c----------------------------------------------------------------------
      if (itd .eq. 0) then
c                           *** no tilt: copy tensors directly
         do 260 j=1,5
         fgd(1,j)=fgm(j)
           do 259 i=1,2
            fad(i,j)=fam(i,j)
  259       continue
  260    continue
c
      else
c                    *** Transform A and g tensors into the diffusion frame
c
        call cd2km(d2km,ald,bed,gad)
        do 265 i=1,5
          do 264 j=1,5
            fgd(1,i)=fgd(1,i)+d2km(1,i,j)*fgm(j)
            fgd(2,i)=fgd(2,i)+d2km(2,i,j)*fgm(j)
            fad(1,i)=fad(1,i)+d2km(1,i,j)*fam(1,j)
     #                       -d2km(2,i,j)*fam(2,j)
            fad(2,i)=fad(2,i)+d2km(1,i,j)*fam(2,j)
     #                       +d2km(2,i,j)*fam(1,j)
 264      continue
 265    continue
      end if
c
c----------------------------------------------------------------------
c     get basis set parameters and verify
c----------------------------------------------------------------------
c
      inquire(file=ixname,exist=fexist)
      if (fexist) then
        go to 275
      else
        ptol=0.0d0
        ndim=0
      end if
c
 266  write(luttyo,2170) lemx,lomx,kmx,mmx,ipnmx
      fprmpt=prmpt(line)
      if (fprmpt) then
        lemx=itoken(line)
        lomx=itoken(line)
        kmx=itoken(line)
        mmx=itoken(line)
        ipnmx=itoken(line)
      end if
c
      lkmtrc = 0
      write (luttyo,2172) lkmtrc
      fprmpt=prmpt(line)
      if (fprmpt) lkmtrc=itoken(line)
      if (lkmtrc.ne.0) then
         lkmtrc=1
         lkmdff = 0
         write(luttyo,2173) lkmdff
         fprmpt=prmpt(line)
         if (fprmpt) lkmdff=itoken(line)
      end if
c
c----------------------------------------------------------------------
c     Check basis set parameters
c----------------------------------------------------------------------
      inlemx = lemx
      inlomx = lomx
      inkmx  = kmx
      inmmx  = mmx
      inpnmx = ipnmx
c
      if((lemx.gt.48).and.(ipt.ne.0)) then
         write (luttyo,5008)
         lemx=48
      endif
c
      if((lemx.gt.mxlval)) lemx=mxlval
      if(ipar(lemx).ne.1) lemx=lemx-1
      if(ipar(lomx).ne.-1) lomx=lomx-1
      if(lomx.gt.lemx) lomx=lemx-1
      if(kmx.gt.lemx) kmx=lemx
      if(mmx.gt.lemx) mmx=lemx
      if(ipar(kmx).ne.1) kmx=kmx-1
      if(ipnmx.gt.in2) ipnmx=in2
      if((ipsi0.eq.0).and.(mmx.gt.ipnmx-ips+1)) mmx=ipnmx
c
      if(lemx.lt.0)  lemx=0
      if(lomx.lt.0)  lomx=0
      if(kmx.lt.0)   kmx=0
      if(mmx.lt.0)   mmx=0
      if(ipnmx.lt.0) ipnmx=0
c
c----------------------------------------------------------------------
c  Determine basis set using rules in Meirovitch,Igner,Igner,Moro & Freed
c  (J. Phys. Chem. 77 (1982) p. 3935)
c----------------------------------------------------------------------
c
c----------------------------------------------------------------------
c Set jmmn=-1 (i.e. use antisymmetric M combinations) if the nuclear
c Zeeman term is non-zero.
c----------------------------------------------------------------------
c
      jmmn=1
      if (abs(zeen).gt.rndoff) jmmn=-1
c
c----------------------------------------------------------------------
c Set jkmn=-1 (i.e. use antisymmetric K combinations) if the magnetic 
c tensors in the diffusion frame are complex-valued. This can result
c if there is magnetic tilt involving alpha_m or gamma_m, or if 
c there is diffusion tilt involving gamma_d.
c----------------------------------------------------------------------
c
      jkmn=1
      do 270 i=1,5
        if (    dabs(fgd(2,i)).ge.rndoff 
     #     .or. dabs(fad(2,i)).ge.rndoff ) jkmn=-1
  270   continue
c
c----------------------------------------------------------------------
c Use only even K values (kdel=2) if there is no magnetic and
c diffusion tilt.
c
c ldel,kdel used as local variables to fix the compiler optimization
c bug in xlf.
c----------------------------------------------------------------------
c
      ldel=ldelta
      kdel=kdelta
c
      if (itm.eq.0 .and.itd.eq.0) then
        kdel=2
      else
        kdel=1
      end if
c
c----------------------------------------------------------------------
c Use only even L values (ldel=2) and no K values (kmx=0)
c in case of axial magnetic tensors, axial potential, 
c and no magnetic/diffusion tilt
c----------------------------------------------------------------------
c
      if (axiala .and. axialg .and. kdel.eq.2 .and. kptmx.eq.0)
     # then
        ldel=2
        lomx=0
        kmx=0
      else
        ldel=1
      end if
c
c----------------------------------------------------------------------
c     Report any changes in basis parameters
c----------------------------------------------------------------------
c
      if (inlemx .ne. lemx .or.
     #    inlomx .ne. lomx .or.
     #    inkmx  .ne. kmx  .or.
     #    inmmx  .ne. mmx  .or.
     #    inpnmx .ne. ipnmx ) then
          write (luttyo,5009) lemx,lomx,kmx,mmx,ipnmx
      endif
c
 275  continue
c
      if (ips.ne.0) then
        ldelta=ldel
        kdelta=kdel
        call fbaso(ixname)
      else
        call fbasd(ixname)
      end if
c
      if (fexist) write(luttyo,2170) lemx,lomx,kmx,mmx,ipnmx
      write (luttyo,4010) ndim,ptol
      if (ndim.gt.mxdim) then
        write (luttyo,4011) 
        goto 266
      end if
c
c----------------------------------------------------------------------
c     get the calculation type and related information
c----------------------------------------------------------------------
c
 280  write (luttyo,2180) itype
      fprmpt=prmpt(line)
      if (fprmpt) itype=itoken(line)
c
      if (itype.lt.0) then
        itype=0
      else if (itype.gt.5) then
        itype=5
      end if
c
      if ((itype.ne.4).and.(ips.eq.0)) wtol=wtol*0.2d0
c
      if ((nort.gt.1).and.(itype.ne.2).and.(itype.ne.4)) then
        write (luttyo,2183)
        go to 280
      end if
c
      if (itype.eq.4) then
        if (ndim.gt.xdim) then
          write (luttyo,2185) xdim
          stop
        end if
        go to 999
      end if
c
      if ((itype.eq.5).and.(ndim.gt.xdimg)) then
        write (luttyo,2185) xdimg
        stop
      end if
c
      if (itype.le.3) then
        write(luttyo,2190) nstep
        fprmpt=prmpt(line)
        if (fprmpt) nstep=itoken(line)
        if (nstep.gt.mxstep) nstep=mxstep
        if (nstep.eq.0) nstep=ndim
      end if
c
      if ( (itype.le.1).or.(itype.eq.3) ) then
        write (luttyo,2195) (nmev(i),i=1,10)
        fprmpt=prmpt(line)
        if (fprmpt) then
          nmev(1)=itoken(line)
          nmev(2)=itoken(line)
          nmev(3)=itoken(line)
          nmev(4)=itoken(line)
          nmev(5)=itoken(line)
          nmev(6)=itoken(line)
          nmev(7)=itoken(line)
          nmev(8)=itoken(line)
          nmev(9)=itoken(line)
          nmev(10)=itoken(line)
        end if
        if (nmev(1).eq.0) nmev(1)=ndim
      end if
c
      if (itype.ge.1) then
c
        if (itype.eq.2) then
 98       write (luttyo,2197) ptol
          fprmpt=prmpt(line)
          if (fprmpt) ptol=ftoken(line)
          if (ptol.gt.0.5D0) then
            write (luttyo,2198)
            go to 98
          end if
        end if
c
        if (itype.eq.3) then
          write (luttyo,2199) mxcg
          fprmpt=prmpt(line)
          if (fprmpt) mxcg=itoken(line)
          if (mxcg.eq.0) mxcg=ndim
        end if
c
        if ((itype.ge.1).and.(itype.le.3)) then
          write (luttyo,2200) cgtol
          fprmpt=prmpt(line)
          if (fprmpt) cgtol=ftoken(line)
          if (cgtol.lt.10.0D0*rndoff) cgtol=10.0D0*rndoff
        end if
c
        if ((itype.ge.1).and.(itype.le.3).or.
     #      ((itype.eq.5).and.(ips.eq.1))) then
          write (luttyo,2210) shiftr,shifti
          fprmpt=prmpt(line)
          if (fprmpt) then
            shiftr=ftoken(line)
            shifti=ftoken(line)
          end if
          if ((itype.ne.5).and.(shiftr.lt.10.0D0*rndoff))
     #        shiftr=10.0D0*rndoff
        end if
c
      else
        shiftr=zero 
        shifti=zero 
      end if
c
      if ((itype.eq.2).or.((itype.eq.5).and.(ips.eq.1))) then
        write (luttyo,2220) b0,fieldi,fieldf
        fprmpt=prmpt(line)
        if (fprmpt) then
          fieldi=ftoken(line)
          fieldf=ftoken(line)
        end if
        write (luttyo,2230) mxnpt+1,nfield
        fprmpt=prmpt(line)
        if (fprmpt) nfield=itoken(line)
        if (nfield.gt.mxnpt) nfield=mxnpt
        if (nfield.lt.2) nfield=2
      end if
c
c----------------------------------------------------------------------
c     check for starting vector for diagonal space
c----------------------------------------------------------------------
c
 999  if ((ips.eq.0).and.(itype.ne.2)) then
        write (luttyo,2240) int1,stt1,nstp
        fprmpt=prmpt(line)
        if (fprmpt) then
          int1=itoken(line)
          stt1=itoken(line)
          nstp=itoken(line)
        end if
        if (nstp.gt.128) then
          write (luttyo,2250)
          go to 999
        end if
        if ((nstp.gt.5).and.(itype.ne.4)) then
          write (luttyo,2255)
          nstp=5
        end if
        write (luttyo,2260) nevo
        fprmpt=prmpt(line)
        if (fprmpt) nevo=itoken(line)
      end if
c
c----------------------------------------------------------------------
c     print out new values of parameters
c----------------------------------------------------------------------
c
      call wrfprm(prname,namlth,luttyo) 
c
      ibasis=0
      write(luttyo,4000) ibasis
      fprmpt=prmpt(line)
      if (fprmpt.and.line(1:1).ne.'n'.and.line(1:1).ne.'N') ibasis=1
c
c----------------------------------------------------------------------
c     write parameters to formatted disk file
c----------------------------------------------------------------------
      open (unit=lulog,file=fmname,status='unknown',
     #      access='sequential',form='formatted')
      rewind (unit=lulog)
c
      call wrfprm(fmname,namlth,lulog)
      if (ibasis.ne.0) call wrfbas(fmname,namlth,lulog)
c
      close (unit=lulog)
c
c----------------------------------------------------------------------
c     write parameters to unformatted disk file (used by EPRLF/EPRCGF)
c----------------------------------------------------------------------
c
      call wrfdat(prname)
c
      stop
c
c======================================================================
c               format statements
c======================================================================
c
 1000 format(//,2x,70('#'),//)
 1010 format(25x,'PROGRAM : LBLF',/,25x,'--------------',//)
 1040 format('* Data file ',a,' already exists on the disk')
 1041 format('* pS from .par file is different.  Assume input value')
c
 2000 format(2x,'g-tensor [gxx,gyy,gzz] : ',3(g13.6,2x))
 2010 format(2x,'twice the nuclear spin [in2] : ',i2)
 2020 format(2x,'A-tensor [axx,ayy,azz] (gauss) : ',3(g13.6,2x))
 2021 format(2x,'Nuclear gyromagnetic ratio [gamman] (s*gauss)^-1 : ',
     #      g13.6)
 2030 format(2x,'static field [B0] (gauss) : ',g13.6)
c
 2040 format(2x,'diffusion parameter [ipdf] : ',i1,/,
     #       10x,'ipdf=0 --> brownian diffusion',/,
     #       10x,'ipdf=1 --> free or jump diffusion',/,
     #       10x,'ipdf=2 --> anisotropic viscosity (if ist=0)')
 2050 format(2x,'diff. tensor [dx,dy,dz] (1/sec) =',3(1p,e14.7,1x))
 2055 format(2x,'weighting factor threshold [wtol] : ',1p,e14.7)
 2060 format(2x,'residence times [tl,tkxy,tkzz] (sec)',/,
     #     5x,'(brownian model-->tl,tkxy,tkzz=0) ',/,2x,3(2x,g13.6))
 2070 format(2x,'residence times [tl,tkxy,tkzz]  (sec)',/,2x,
     #     8x,3(2x,g13.6))
 2073 format(2x,'perpendicular diffusion coefficient in the director',
     #     /,5x,'frame [djfprp] (1/sec) : ',g13.6)
 2080 format(2x,'non-Brownian exponents [pl,pkxy,pkzz] : ',/,
     #     10x,'0.5 --> free',/,10x,'1.0 --> jump',/,
     #     2x,3(f6.2,2x))
 2090 format(2x,'discrete jumps motion [ist,djf] : ',
     #     i2,2x,g13.6,/,4x,
     #     '(if ipdf=2 then ist=0 and djf is the parallel',/,4x,
     #     'diffusion coefficient in the director frame)')
c
 2100 format(2x,'Heisenberg spin exchange frequency [oss] = ',g13.6)
c
 2105 format(2x,'number of orientations for MOMD [nort] = ',i3,/,2x,
     #      '(nort=0 -> ordinary calc.; nort>0 -> MOMD calc.)')
c 2107 format(2x,'** Nuclear Zeeman with MOMD is not active option ',
c     #      'yet **',/,5x,'It needs separate diagonalization for ',
c     #      'pS=-1 subspace.',/,2x,'** Nuclear Zeeman ignored **',/)
 5200 format(5x,'*** Nuclear Modulation with MOMD is not active ',
     #     'option yet ***',/,9x,'Nuclear Zeeman term is ignored!!')
 2110 format(2x,'number of terms in the potential [ipt] = ',i1)
 2120 format(2x,'coefficients of the potential : ')
 2130 format(10x,'ipt = ',i1,2x,'[l,k,coef.] = ',2(i1,2x),g13.6)
 2140 format(2x,'angle between B0 and local director [psi] ',
     #     '(degrees) : ',g13.6)
c
 2160 format(2x,'diffusion tilt [ald,bed,gad] (degrees) = ',
     #       2(g10.3,2x),g10.3)
 2162 format(2x,'magnetic tilt [alm,bem,gam] (degrees) = ',
     #       2(g10.3,2x),g10.3)
c
 2165 format(2x,'1/T20e, 1/T20n, 1/T22e (1/sec) = ',1p,e10.3,
     #	     2x,1p,e10.3,2x,1p,e10.3)
 2166 format(2x,7x,'1/T1e, 1/T1n (1/sec) = ',7x,1p,e10.3,2x,1p,e10.3)
c
 2170 format(2x,'truncation values  [lemx,lomx,kmx,'
     #     ,'mmx,ipnmx] : ',5(i3,1x))
c
 2172 format(2x,'L+K+M truncation rule flag [lkmtrc] : ',i2)
 2173 format(2x,'L+K+M truncation bandwidth [lkmdff] : ',i3)
 2180 format(2x,'cal. type (0=L,1=CG,2=FS,3=LCG,4=ESA,5=EGV)',
     #      ' [itype] : ',i1)
 2183 format(2x,'** MOMD calc. is valid only for FS or ESA ',
     #      'algorithm **')
 2185 format(2x,'*** maximum dimension for ESA calculation is : ',
     #      i4,' ***',/,6x,'reduce the matrix size or modify ',
     #      'lblf and epresa programs.')
 2190 format(2x,'number of Lanczos/CG steps [nstep] : ',i4)
 2195 format(2x,'steps to check the convergence [nmev] : ',/,
     #     10x,10i5) 
 2197 format(2x,'pruning tolerance [ptol] : ',1p,e10.4)
 2198 format(2x,'** invalid pruning tolerance (ptol > 0.5) **')
 2199 format(2x,'# of CG steps in eigenvector cal. [mxcg] : ',i4)
 2200 format(2x,'error tolerance for residual [cgtol] : ',g13.4)
 2210 format(2x,'origin shift [shiftr,shifti] :',2(2x,g13.6))
 2220 format(2x,'initial and final fields relative to B0=', g13.6,
     #     /,2x,'[fieldi,fieldf] (gauss) : ',g13.6,2x,g13.6)
 2230 format(2x,'number of field positions [nfield<',i3,'] : ',i3)
 2240 format(2x,'starting vector [int1,stt1,nstp] : ',i4,2x,i3,2x,i3)
 2250 format(2x,'*** maximum nstp allowed is 128 ***')
 2255 format(2x,'*** Only 5 t1 values are used in evaluating starting ',
     #     'vector ***')
 2260 format(2x,'# of significant eigenvalues in off-diagonal ',
     #     'space [nevo] : ',i4)
c
 4000 format(2x,'List the basis set indices [ibasis] : ',i1)
 4010 format(2x,'** Dimension of matrix =',i4,'  with ptol = ',
     #       g10.4,/)
 4011 format(/,18x,'************* WARNING ****************',
     #     /,18x,'BASIS SET TOO LARGE FOR PRESENT ARRAYS',
     #     /,18x,'**************************************',//
     #     2x,'Please respecify basis truncation parameters'/)
 5004 format(' Times for jump/free model are all zero:',
     #       ' ipdf=0 assumed')
 5005 format(' Nonzero potential with jump/free model:',
     #       ' ipdf=0 assumed') 
 5006 format(' Zero potential with anisotropic viscosity:',
     #       ' ipdf=0 assumed')
 5007 format(' Discrete jumps with anisotropic viscosity:',
     #       ' ipdf=0 assumed')
 5008 format(' lemx must be 48 or less with potential;',
     #       ' lemx=48 assumed')
 5009 format(' *Basis set parameters adjusted to [lemx,lomx,',
     #       'kmx,mmx,ipnmx] =',/,10x,4(i3,','), i3)
 5014 format(' Angle alm is not needed in the case of an axial',
     #       ' g-tensor')
 5015 format(' Angles alm,gam,and ged are not needed for axial',
     #       ' magnetic tensors')
c
c======================================================================
c
      end
