c**********************************************************************
c
c                       ==============
c                       PROGRAM : TDLF
c                       ==============
c
c       This program processes the tridiagonal matrix generated 
c       by the Lanczos or conjugate gradients algorithms.
c
c       written by DJS 29-AUG-87
c
c       UPDATED 25-OCT-91 by DEB to calculate derivative spectrum
c       directly from continued fraction using expressions given in
c       G. Moro and J.H. Freed, J. Chem. Phys. 74 (1981) 3757.
c
c       updated 3-15-92 by DEB to use new filename convention.
c
c       Includes:
c               stddim.inc
c               stdio.inc
c               eprdat.inc
c               fnames.inc
c               pidef.inc
c               rndoff.inc
c
c       Uses:
c               getids.f
c               setnam.f
c               prmpt.f
c               rdfdat.f
c               cmtqli.f
c
c**********************************************************************
c
      program tdlf
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'fnames.inc'
      include 'pidef.inc'
      include 'rndoff.inc'
c
      integer mxpt
      parameter (mxpt=2048)
      double precision zero,unity,two
      parameter (zero=0.0D0,unity=1.0D0,two=2.0D0)
c
      integer i,j,iopt,nstepl,ierr,diag,npts,nfiles,isort
c
c --- modified DEB ---
      double precision bi,bf,betr,beti,b2r,b2i,bsweep,db,rtemp,
     #   scale,scale2,sprodr,sprodi,tr,ti,termr,termi,twob0,wint,
     #   yr,yi,zr,zi,zr2,zi2,z2r,z2i
c ---
c
      double precision alpha,beta,spect,field,w
      dimension alpha(2,mxstep),beta(2,mxstep),field(mxpt),
     #     spect(2,mxpt),w(2,mxstep)
      double precision temp1,temp2
      dimension temp1(2,mxstep),temp2(2,mxstep)
c
      logical fprmpt
      integer itoken
      character*80 line
c
      double precision ftoken
      logical prmpt
      external prmpt,itoken,ftoken
c
c######################################################################
c
      write (luttyo,1010)
      write (luttyo,1000)
c
      call getids(nfiles,1)
      if (nfiles.eq.0) stop
      fileid=files(1)
      ips=idxps(1)
      call setnam
c
c----------------------------------------------------------------------
c     read in data from parameter file
c----------------------------------------------------------------------
c
        call rdfdat(prname,ierr)
        if (nstep.eq.0) nstep=ndim
c
c----------------------------------------------------------------------
c     read in tridiagonal matrix
c----------------------------------------------------------------------
c
        open (unit=ludisk,file=tdname,status='old',
     #       access='sequential',form='unformatted')
        read (ludisk) (alpha(1,i),alpha(2,i),beta(1,i),beta(2,i),
     #       i=1,nstep)
        close (unit=ludisk)
c    zero out the last beta element
c        beta(1,nstep)=zero
c        beta(2,nstep)=zero
c
c----------------------------------------------------------------------
c     prompt user for processing option
c----------------------------------------------------------------------
c
 25     write (luttyo,1040) tdname(:namlth)
        read (luttyi,1050,err=25) iopt
c
c----------------------------------------------------------------------
c     list matrix elements of screen
c----------------------------------------------------------------------
c
        if (iopt.eq.1) then
          write (luttyo,1060) tdname(:namlth)
          do 20 i=1,nstep
            write (luttyo,1070) i,alpha(1,i),alpha(2,i),
     #           beta(1,i),beta(2,i)
 20       continue
c
c----------------------------------------------------------------------
c     write formatted tridiagonal matrix file
c----------------------------------------------------------------------
c
        else if (iopt.eq.2) then
          open (unit=ludisk,file=tfname,
     #         access='sequential',form='formatted')
          write (ludisk,1080) (dble(i),alpha(1,i),alpha(2,i),
     #         beta(1,i),beta(2,i),i=1,nstep)
          close (unit=ludisk)
c
c----------------------------------------------------------------------
c     diagonalize tridiagonal matrix 
c----------------------------------------------------------------------
c
        else if (iopt.eq.3) then
          write(luttyo,1150) nstep+1
          read(luttyi,*) diag
          if (diag.gt.nstep) diag=nstep
          do 30 i=1,diag
            temp1(1,i)=alpha(1,i)
            temp1(2,i)=alpha(2,i)
 30       continue
c
          do 40 i=1,diag
            temp2(1,i)=beta(1,i)
            temp2(2,i)=beta(2,i)
 40       continue
c       
          temp2(1,diag)=zero
          temp2(2,diag)=zero
c
          isort=1
          call cmtqli(diag,temp1,temp2,w,isort,ierr)
c
          if (ierr.ne.0) write(luttyo,1140) ierr
c
          open (unit=ludisk,file=egname,
     #         access='sequential',form='formatted')
          write (ludisk,1130) (i,0,temp1(1,i),temp1(2,i),
     #         w(1,i),w(2,i),i=1,diag)
          close (unit=ludisk)
c
c----------------------------------------------------------------------
c     calculate spectrum from continued fraction
c----------------------------------------------------------------------
c
        else if (iopt.eq.4) then
          bi=-50.0D0
          bf=50.0D0
          write(luttyo,1090) b0,bi,bf
          fprmpt=prmpt(line)
          if (fprmpt) then
             bi=ftoken(line)
             bf=ftoken(line)
          end if
c
          wint0=unity
          wint2=zero
          write(luttyo,1100) wint0,wint2
          fprmpt=prmpt(line)
          if (fprmpt) then
             wint0=ftoken(line)
             wint2=ftoken(line)
          end if
c
          wint=wint0
          if (ipt.ne.0) wint=wint0+wint2*(dcos(psi*pi/180.0D0))**2
c
          nstepl=nstep
          write(luttyo,1110) nstep
          fprmpt=prmpt(line)
          if (fprmpt) nstepl=itoken(line)
c
c *** Modification: specify whether derivative or not
c
          ideriv = 0
          write(luttyo,1115) ideriv
          fprmpt=prmpt(line)
          if (fprmpt) ideriv=itoken(line)
c
          npts=512
          write(luttyo,1116) mxpt,npts
          fprmpt=prmpt(line)
          if (fprmpt) npts=itoken(line)
          if(npts.lt.1) npts=1
          if(npts.gt.mxpt) npts=mxpt
c
c *** Note that field sweep proceeds from high to low
c     so that final spectrum will appear in low-to-high order
c     after transposition about b0 (cf. below)
c
          db=(bi-bf)/dble(npts-1)
          bsweep=b0-bi
          twob0 = b0+b0
          do 80 i=1,npts
            spect(1,i)=zero
            spect(2,i)=zero
            yr=zero
            yi=zero
            sprodr=zero
            sprodi=zero
c
            do 90 j=nstepl,1,-1
              if(j.eq.1) then
                b2r=unity
                b2i=zero
              else
                betr=beta(1,j-1)
                beti=beta(2,j-1)
                b2r=betr*betr-beti*beti
                b2i=two*betr*beti
              end if
c
              zr=alpha(1,j)+wint   -yr
              zi=alpha(2,j)+bsweep -yi
              zr2=zr*zr
              zi2=zi*zi
              scale=unity/(zr2+zi2)
c
              if (ideriv.ne.0) then
c                                    Accumulate sum of products for first
c                                    derivative spectrum
                scale2=scale*scale
                z2r=zr2-zi2
                z2i=-two*zr*zi
                termr=(b2r*z2r-b2i*z2i)*scale2
                termi=(b2i*z2r+b2r*z2i)*scale2
                rtemp=sprodr
                sprodr=(sprodr*termr-sprodi*termi)+termr
                sprodi=(rtemp*termi+sprodi*termr)+termi
              end if
              yr=scale*(b2r*zr+b2i*zi)
              yi=scale*(b2i*zr-b2r*zi)
 90         continue
c
            if (ideriv.ne.0) then
              spect(1,i) = -sprodi/pi
              spect(2,i) = -sprodr/pi
            else
              spect(1,i)=yr/pi
              spect(2,i)=yi/pi
            end if
c
c  Transpose spectrum around B0 because we have calculated a frequency
c  spectrum but want to represent a field-swept spectrum
c
            field(i)=twob0-bsweep
            bsweep=bsweep+db
 80       continue
c
c *** end of revisions (DEB)
c
c
c----------------------------------------------------------------------
c     write spectrum to disk
c----------------------------------------------------------------------
c
          open (unit=ludisk,file=spname,status='unknown',
     #         form='formatted')
          write (ludisk,1120)(field(i),spect(1,i),spect(2,i),
     #         i=1,npts)
          close (unit=ludisk)
c
        else if (iopt.eq.5) then
          go to 9999
        end if
c
        go to 25
c
c----------------------------------------------------------------------
c     exit program
c----------------------------------------------------------------------
c
 9999   write(luttyo,1010)
        stop
c
c======================================================================
c     format statements
c======================================================================
c
 1000   format(25x,'PROGRAM : TDLF',/,25x,'--------------',//)
 1010   format(//,70('#'),//)
 1040   format(//,25x,'file : ',a,/,
     #       /,5x,'1 - list elements of tridiagonal matrix on screen',
     #       /,5x,'2 - write tridiagonal matrix into formatted file',
     #       /,5x,'3 - diagonalize tridiagonal matrix',
     #       /,5x,'4 - generate cw spectrum from continued fraction',
     #       /,5x,'5 - exit program',
     #       /,10x,'please select option : ')
 1050   format(i1)
 1060   format(//,25x,'file : ',a,//,
     #       ' no.',16x,'alpha',32x,'beta',/,70('-'))
 1070   format(i3,2(4x,2(2x,g14.7)))
 1080   format(5(g14.7,','))
 1090   format(//,2x,'Please enter initial and final fields',
     #       ' ( B0 = ',f8.2,' )',/,2x,
     #       '[ bi = ',f8.2,', bf = ',f8.2,' ]')
 1100   format(//,2x,'Please enter 1/T20 and 1/T22',/,2x,
     #       '[ wint0 = ',g10.5,', wint2 = ',g10.5,' ]')
 1110   format(//,2x,'Please enter number of Lanczos steps to use',
     #       ' for calculation',/,2x,'[ nstep =',i4,' ]')
 1115   format(//,2x,'Please specify 0 or 1 for 0th or 1st',
     #       ' derivative spectrum',/2x,'[ ideriv =',i2, ' ]' )
 1116   format(//,2x,'Please specify number of points in spectrum ',
     #       '( <',i5,')'/,2x,'[ npts =',i5,' ]')
 1120   format(3(g14.7,','))
 1130 format(i4,2x,i4,4x,g14.7,',',g14.7,2x,g14.7,',',g14.7)
 1140   format(//,10x,'*** ERROR IN DIAGONALIZATION, IERR=',i3,' ***')
 1150   format(/,2x,'Dimension of matrix to be diagonalized ',
     #       '[ndiag<',i3,'] : ')
c
c======================================================================
c
        end
