c*********************************************************************
c
c                       ================
c                       PROGRAM : DUALQD
c                       ================
c
c This program generates dual quadrature signal in time domain to mimic
c the experimental data set.
c
c It reads the Sc+ and/or Sc- time domain signal in the file <id>.fid,
c generated by EPRPS or MOMD program with -t flag.  If two file names
c are supplied, they are assumed to be Sc+ and Sc- signals and used to
c calculate the real and imaginary parts of dual quadrature signal
c along t1 domain (S' and S").
c
c The output data is written in experimental data file format.
c
c       Includes :
c               stdio.inc
c
c       Uses :
c
c*********************************************************************
c
      program dualqd
c
      implicit none
c
      include 'stdio.inc'
c
      logical fexist
c
      character*30 namei1,namei2,nameo
c
      integer i,j,j2,j21,ierr,iend,npt1,npt2
      double precision half
      complex*8 xs(512,512)
      complex*16 xsp(256,512),xsm(256,512),xsr(256,512),xsi(256,512),
     #           chalf
      parameter (half=0.5D0,chalf=(0.0D0,0.5D0))
c
      integer whris
      external whris
c
c#####################################################################
c
      write (luttyo,1000)
      write (luttyo,1010)
c
 15   write (luttyo,1020)
      read (luttyi,1025,err=15) namei1
 25   write (luttyo,1030)
      read (luttyi,1025,err=25) namei2
 35   write (luttyo,1040)
      read (luttyi,1025,err=35) nameo
      write (luttyo,1050)
      read (luttyi,*) npt1,npt2
c
c---------------------------------------------------------------------
c     construct file names
c---------------------------------------------------------------------
c
      iend=whris(namei1,' ')-1
      namei1=namei1(:iend)//'.fid'
      iend=whris(namei2,' ')-1
      namei2=namei2(:iend)//'.fid'
      iend=whris(nameo,' ')-1
      nameo=nameo(:iend)//'.dat'
c
      inquire(file=namei1,exist=fexist)
      if (fexist) then
        open (unit=ludisk,file=namei1,status='old',
     #        access='sequential',form='unformatted')
        do 50 j=1,npt2
 50        read(xsp(i,j),i=1,npt1)
        close (unit=ludisk)
      else
        write (luttyo,1100) namei1
        go to 999
      end if
c
      inquire(file=namei2,exist=fexist)
      if (fexist) then
        open (unit=ludisk,file=namei2,status='old',
     #        access='sequential',form='unformatted')
        do 60 j=1,npt2
 60        read (ludisk) (xsm(i,j),i=1,npt1)
        close (unit=ludisk)
      else
        write (luttyo,1100) namei2
        go to 999
      end if
c
c---------------------------------------------------------------------
c     Calculate dual quadrature signal
c---------------------------------------------------------------------
c
      do 80 i=1,npt2
      do 80 j=1,npt1
 80      xsr(j,i)=(xsm(j,i)+xsp(j,i))*half
      do 90 i=1,npt2
      do 90 j=1,npt1
 90      xsi(j,i)=(xsm(j,i)-xsp(j,i))*chalf
c
      do 100 j=1,npt1
        j2=2*j
        j21=j2-1
        do 100 i=1,npt2
          xs(j21,i)=dconjg(xsr(j,i))
          xs(j2,i)=dconjg(xsi(j,i))
 100  continue
c
      open (unit=ludisk,file=nameo,status='unknown',
     #          access='sequential',form='unformatted')
      do 120 j=1,npt2
 120     write (ludisk) (xs(i,j),i=1,2*npt1)
c
      close (unit=ludisk)
c
 999  continue
c
      stop
c
c=====================================================================
c     format statements
c=====================================================================

c
 1000 format(/,2x,70('#'),/)
 1010 format(25x,'PROGRAM : DUALQD',/,25x,'----------------')
 1020 format(2x,'Enter file id of the Sc+ fid file : ',$)
 1025 format(a)
 1030 format(2x,'Enter file id of the Sc- fid file : ',$)
 1040 format(2x,'Enter file id of the output data file : ',$)
 1050 format(2x,'# of data points [npt1,npt2] : ',$)
 1100 format(/,20x,'*** CANNOT FIND FILE ',a,' ***',/)
c
c=====================================================================
c
      end
