c*********************************************************************
c
c                       ================
c                       PROGRAM : EPRESA
c                       ================
c
c	This program calculates the eigenvalues and eigenvectors
c	for the general complex matrix using Rutishauser Algorithm,
c       described by R. G. Gordon and T. Messenger in "Electron-Spin
c       Relaxation in Liquids", L. T. Muus and P. W. Atkins, (1972).
c
c       Includes :
c               stddim.inc
c               stdio.inc
c               rndoff.inc
c               eprdat.inc
c               eprmat.inc
c               indexf.inc
c               timer.inc
c               fnames.inc
c
c       Uses :
c               getids.f
c               setnam.f
c               setflg.f
c               rdfdat.f
c               wrfdat.f
c               wrfprm.f
c               matrxo.f
c               matrxd.f
c               stvec.f
c               csqz.f
c               cqrt.f
c               indexx.f
c
c*********************************************************************
c
      program epresa
c
      implicit none
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'rndoff.inc'
      include 'eprdat.inc'
      include 'eprmat.inc'
      include 'indexf.inc'
      include 'fnames.inc'
      include 'timer.inc'
c
      character*5 flgstr
      logical fexist 
c
      integer i,j,k,m,n,ncalc,ierr,jcol,kcol,ipsold,indx,t1indx
      integer bdw,bdwtmp,mxit,nstpold
c
      double precision zr,zi,zero,wr,sqtol,tol
c
      complex*16 czero,cone,cmat,evec,x,wt,w,cwsp,stv
      parameter (zero=0.D0,czero=(0.D0,0.D0),cone=(1.D0,0.D0))
      dimension cmat(xdim,xdim),evec(xdim,xdim),cwsp(xdim),
     #          stv(xdim,mxnpt),x(xdim),w(xdim),wr(xdim),
     #          indx(xdim),t1indx(xdim)
c
c
c#####################################################################
c
c----------------------------------------------------------------------
c Timing call changed for compatibility with UNIX
c      call cputime(cpu0,ircode)
c----------------------------------------------------------------------
      cputot = dtime( tarray )
c
      write (luttyo,1000)
      write (luttyo,1010)
c
      call getids(n,mxcalc)
c
c======================================================================
c     loop over calculations
c======================================================================
c
      do 900 ncalc=1,n
c
c---------------------------------------------------------------------
c     construct file names
c---------------------------------------------------------------------
c
      fileid = files(ncalc)
      ips=idxps(ncalc)
      call setnam
      flgstr = flags(ncalc)
      call setflg(flgstr)
c
c----------------------------------------------------------------------
c     open log file and write out header 
c----------------------------------------------------------------------
c
      open (unit=lulog,file=rlname,access='sequential',
     #      form='formatted')
c
      rewind (unit=lulog)
c
      write (luttyo,1000)
      write (lulog,1000)
      write (luttyo,5000) ncalc,prname(:namlth)
      write (lulog,5000) ncalc,prname(:namlth)
c
c----------------------------------------------------------------------
c     read in parameters from parameter file
c----------------------------------------------------------------------
c
      inquire(file=prname,exist=fexist)
c
      ipsold=ips      
      if (fexist) then
        call rdfdat(prname,ierr)
        ips=ipsold
      else
        write (luttyo,1100) prname(:namlth)
        write (lulog,1100) prname(:namlth)
        go to 999
      endif
c
      if (ierr.eq.1) then
        write (luttyo,1110) prname(:namlth)
        write (lulog,1110) prname(:namlth)
        go to 999
      else if (ierr.ne.0) then
        write (luttyo,1120) prname(:namlth)
        write (lulog,1120) prname(:namlth)
        go to 999
      end if
c
c----------------------------------------------------------------------
c     check to make sure that Exact Eigen System analysis desired
c----------------------------------------------------------------------
c
      if (itype.ne.4) then
        write (luttyo,5100) 
        write (lulog,5100)
        go to 999
      end if
c
      if (nort.gt.1) then
        write (luttyo,5200) 
        write (lulog,5200)
        go to 999
      end if
c
c---------------------------------------------------------------------
c	print out parameter set
c---------------------------------------------------------------------
c
      call wrfprm(prname,namlth,lulog)
c
c---------------------------------------------------------------------
c     calculate the matrix elements
c---------------------------------------------------------------------
c
      write(luttyo,1040)
      write(lulog,1040) 
c
      if (ips.ne.0) then
        call fbaso(ixname)
        if (ndim.gt.xdim) then
          write (luttyo,1060)
          write (lulog,1060)
          go to 999
        end if          
        call matrxo(ierr)
      else
        call fbasd(ixname)
        if (ndim.gt.xdim) then
          write (luttyo,1060)
          write (lulog,1060)
          go to 999
        end if          
        call matrxd(ierr)
      end if
c
      if (ierr.ne.0) then
        write (luttyo,1070) 
        write (lulog,1070)
        if (ierr.eq.1) then
          write (luttyo,1080)
          write (lulog,1080) 
        end if
        go to 999
      end if
c                                     ** store matrix **
      if (stmflg) then 
        open (unit=ludisk,file=mtname,status='unknown',
     #       access='sequential',form='unformatted')
        write (luttyo,4000) mtname(:namlth)
        write (lulog,4000)  mtname(:namlth)
        rewind (unit=ludisk)
        write (ludisk) (zdiag(1,i),zdiag(2,i),i=1,ndim)
        write (ludisk) (jzmat(i),i=1,ndim+1) 
        write (ludisk) (izmat(i),zmat(i),i=1,nelim)
        write (ludisk) (kzmat(i),i=1,ndim+1)
        write (ludisk) (izmat(mxel-i+1),zmat(mxel-i+1),i=1,nelre)
        close (unit=ludisk)
      end if
c
 100  cpu1 = dtime( tarray )
      write (luttyo,1250) ndim,neltot,cpu1
      write (lulog,1250) ndim,neltot,cpu1
c
c---------------------------------------------------------------------
c     starting vector calculation
c---------------------------------------------------------------------
c
      nstpold=nstp
      call stvec(stv,x,cwsp,evec,xdim,0)
c                              ** store starting vector **
      if ( (ips.eq.1).or.stvflg ) then
        open (unit=ludisk,file=vtname,status='unknown',
     #        access='sequential',form='unformatted')
        write (luttyo,4000) vtname(:namlth)
        write (lulog,4000)  vtname(:namlth)
        write (ludisk) (stv(i,1),i=1,ndim)
        close (unit=ludisk)
      end if
c
      cpu2 = dtime( tarray )
c
      if (ips.eq.1) then      
          write (luttyo,2020) nelv,cpu2
          write (lulog,2020) nelv,cpu2
      end if
c
c---------------------------------------------------------------------
c     convert the matrix into banded storage format
c---------------------------------------------------------------------
c
      if (evalflg) then
c				**  read tridiagonal matrix **
        inquire(file=tdname,exist=fexist)
        if (fexist) then
          write (luttyo,2100)
          write (lulog,2100)
          open (unit=ludisk,file=tdname,status='old',
     #          access='sequential',form='unformatted')
          read(ludisk) (cmat(j,1),cmat(j,2),j=1,ndim)
          close (unit=ludisk)
          go to 200
        end if
      end if
c
      write (luttyo,2200)
      write (lulog,2200)
c
      do 105 j=1,xdim
      do 105 i=1,ndim
 105    cmat(i,j)=czero
c
      j=1
      k=1
c
      bdw=0
      do 110 m=1,ndim
        if ((abs(zdiag(1,m)).gt.rndoff).or.
     #      (abs(zdiag(2,m)).gt.rndoff)) then
          cmat(m,1)=dcmplx(zdiag(1,m),zdiag(2,m)) 
        endif
c
        if (jzmat(m).ne.jzmat(m+1)) then
          jcol=izmat(jzmat(m+1)-1)
        else
          jcol=m
        end if
c
        if (kzmat(m).ne.kzmat(m+1)) then
          kcol=izmat(mxel-kzmat(m+1)+2)
        else
          kcol=m
        end if
c
        bdwtmp=max(jcol,kcol)-m+1        
        if (bdwtmp.gt.bdw) bdw=bdwtmp
c
        do 120 n=m+1,max(jcol,kcol)
        if ((n.le.jcol).and.(n.eq.izmat(j))) then
          zi=zmat(j)
          j=j+1
        else
          zi=zero
        endif
        if ((n.le.kcol).and.(n.eq.izmat(mxel-k+1))) then
          zr=zmat(mxel-k+1)     
          k=k+1
        else
          zr=zero
        endif
c
        if ((abs(zr).gt.rndoff).or.(abs(zi).gt.rndoff)) then
          cmat(m,n-m+1)=dcmplx(zr,zi)
        endif
c
 120    continue
c
      if ((j.ne.jzmat(m+1)).or.(k.ne.kzmat(m+1))) then
        write (luttyo,2220)
        stop
      endif
c
 110  continue
c
      if (bdw.gt.xdim) then
          write (luttyo,1200) bdw
          stop
      end if
      write (luttyo,2230) bdw
      write (lulog,2230) bdw
c
c---------------------------------------------------------------------
c     convert the complex symmetric matrix into tridiagonal matrix
c---------------------------------------------------------------------
c
      sqtol=zero
c
      do 140 j=1,ndim
      do 140 i=1,ndim
 140    evec(i,j)=czero
c
      do 145 i=1,ndim
 145    evec(i,i)=cone
c
      call csqz(cmat,evec,xdim,xdim,xdim,xdim,ndim,bdw,sqtol)
c
 200  cpu3 = dtime( tarray )
      write (luttyo,2250) cpu3
      write (lulog,2250) cpu3
c                             ** store tridiagonal matrix **
c
      if (tdlflg) then
        write (luttyo,4000) tdname(:namlth)
        write (lulog,4000) tdname(:namlth)
c
        open (unit=ludisk,file=tdname,status='unknown',
     #       access='sequential',form='unformatted')
        rewind (unit=ludisk)
        write(ludisk) (cmat(j,1),cmat(j,2),j=1,ndim)
        close (unit=ludisk)
      end if
c
c---------------------------------------------------------------------
c     Solve tridiagonal matrix for eigenvalues and eigenvectors
c---------------------------------------------------------------------
c
      write (luttyo,2300)
      write (lulog,2300)
c
      mxit=100
c
      do 160 i=1,ndim
 160    cwsp(i)=czero
c
      call cqrt(cmat(1,1),cmat(1,2),evec,ndim,xdim,xdim,
     #          mxit,tol,cwsp,ierr)
c
      cpu4 = dtime( tarray )
      write (luttyo,2250) cpu4
      write (lulog,2250) cpu4
c 
      if (ierr.ne.0) then
        write (luttyo,2340) mxit
        call exit
      end if
c
c---------------------------------------------------------------------
c     calculate weighting of each eigenvector on the starting vector
c---------------------------------------------------------------------
c
      if (ips.ne.0) then
c				*** off-diagonal space
        do 300 i=1,ndim
          wt=czero
          do 301 j=1,ndim
 301        wt=wt+evec(j,i)*stv(j,1)
          w(i)=wt*wt
          wr(i)=cdabs(w(i))
 300    continue
      else
c				*** diagonal space
c     convert the eigenvectors of the diagonal space into the
c     corresponding off-diagonal space using the pulse propagator
c
        do 350 i=1,ndim
          k=1
          do 360 j=1,ndimo
            if (pid(j).eq.1) then
              evec(j,i)=-pp(k)*evec(k,i)
              k=k+1
            else
              evec(j,i)=-pp(k)*evec(k,i)-pp(k+1)*evec(k+1,i)
              k=k+2
            end if
 360      continue
c
          wr(i)=zero
          w(i)=czero
          do 370 j=1,nstp
            wt=czero
            do 375 k=1,ndimo
 375          wt=wt+evec(k,i)*stv(k,j)
            wt=wt*wt
            if (cdabs(wt).gt.wr(i)) then
              w(i)=wt
              wr(i)=cdabs(wt)
              t1indx(i)=j
            end if
 370      continue
 350    continue
c
      end if
c
      call indexx(ndim,wr,indx)
c
c---------------------------------------------------------------------
c     store the significant eigenvalues and eigenvectors
c---------------------------------------------------------------------
c
      write (luttyo,2270)
      write (lulog,2270)
c
      open (unit=ludisk,file=egname(:namlth),status='unknown',
     #     access='sequential',form='formatted')
      write (ludisk,2420) (i,t1indx(indx(i)),cmat(indx(i),1),
     #     w(indx(i)),i=1,ndim)
      close (unit=ludisk)
c
      ngood=ndim
c
      open (unit=ludisk,file=evname(:namlth),status='unknown',
     #     access='sequential',form='unformatted')
      nev=0
      do 400 i=1,ndim
        if (wr(indx(i)).lt.wtol) go to 401
        nev=nev+1
        write (ludisk) (evec(j,indx(i)),j=1,ndimo)
 400  continue
 401  close (unit=ludisk)
c
      write (luttyo,4310) nev,ndim
      write (lulog,4310) nev,ndim
c
      cpu5 = dtime( tarray )
c
c---------------------------------------------------------------------
c     update parameter file
c---------------------------------------------------------------------
c
      write (luttyo,4000) prname(:namlth)
      write (lulog,4000) prname(:namlth)
c
      if (ips.eq.1) nevo=nev
      if (nstpold.ne.nstp) nstp=nstpold
      call wrfdat(prname)
c
 999  continue
c
c---------------------------------------------------------------------
c     print out trailer and close log file
c---------------------------------------------------------------------
c
      write (luttyo,1000)
      write (lulog,1000)
c
      cputot = cputot+cpu1+cpu2+cpu3+cpu4+cpu5
c
      write (lulog,6000) cputot
      write (luttyo,6000) cputot
      close (unit=lulog)
c
c======================================================================
c     end of loop over calculations
c======================================================================
c
 900  continue
c
c---------------------------------------------------------------------
c     exit program
c---------------------------------------------------------------------
c
      stop
c
c=====================================================================
c     format statements
c=====================================================================
c
 1000 format(/,2x,70('#'),/)
 1010 format(25x,'PROGRAM : EPRESA',/,25x,'----------------',/)
c
 1040 format(/,15x,'*** Calculating matrix elements ***')
 1060 format(/,20x,'*** MATRIX DIMENSION TOO LARGE ***',/,
     #     4x,'Use program compiled with bigger dimension or reduce',
     #     ' the dimension.')
 1070 format(//,10x,
     #     '*** SERIOUS ERROR ENCOUNTERED IN MATRIX GENERATION ***')
 1080 format(/,20x,'*** TOO MANY OFF-DIAGONAL MATRIX ELEMENTS ***')
 1100 format(/,20x,'*** CANNOT FIND FILE ',a,' ***',/)
 1110 format(/,20X,'*** FILE ',a,' NOT EPRLF ***',/)
 1120 format(/,20X,'*** ERROR READING FILE ',a,' ***',/)
 1200 format(/,20X,'*** BANDWIDTH : ',i4,' IS TOO WIDE!! ***',/)
 1250 format(/,2x,'Dimension of the matrix : ',i5,
     #       /,2x,'Total number of non-zero off-diagonal entries : ',i6,
     #       /,2x,'CPU time (seconds) : ',f10.2,/)
c
 2020 format(/,2x,'Number of non-zero elements in ',
     #     'starting vector : ',i5,
     #     /,2x,'CPU time (seconds) : ',f10.2)
c
 2100 format(/,15x,'*** Reading tridiagonal matrix ***')
 2200 format(/,15x,'*** Calculating tridiagonal matrix ***')
 2220 format(//,15x,'*** OOPS!, MATRIX FILE CORRUPTED ***',//)
 2230 format(/,2x,'BandWidth = ',i4)
 2250 format(/,2x,'CPU time (seconds) : ',f10.2)
 2300 format(/,15x,'*** Diagonalizing tridiagonal matrix ***')
 2320 format(i4,2x,i4,4x,g14.7,1x,g14.7,2x,g14.7,1x,g14.7)
 2340 format(/,5x,'!!! CQRT has not converged in',i4,' iterations !!!')
 2270 format(/,15x,'*** Writing eigenvalues and eigenvectors ***')
 2420 format(i4,2x,i4,4x,g14.7,',',g14.7,2x,g14.7,',',g14.7)
c
 4000 format(/,20x,'*** Writing ',a,' ***')
 4310 format(/,2x,i4,' eigenvectors stored out of ',i4)
 5000 format(15x,38('='),/,
     #     15x,'Calculation no. ',I2,' using file ',a,/,
     #     15x,38('='),/)
 5100 format(/,10x,'*** NOT EXACT EIGENSYSTEM ANALYSIS ***')
 5200 format(/,10x,
     #     '*** USE PROGRAM MOMD FOR ORIENTATIONAL AVERAGE***')
c
 6000 format(/,2x,'Total CPU time (seconds) : ',f10.2)
c
c=====================================================================
c
      end
