c*********************************************************************
c
c                       ==================
c                       SUBROUTINE : EVESA
c                       ==================
c
c	This subroutine calculates the eigenvalues and eigenvectors
c	for the general complex matrix using Rutishauser Algorithm,
c       described by R. G. Gordon and T. Messenger in "Electron-Spin
c       Relaxation in Liquids", L. T. Muus and P. W. Atkins, (1972).
c
c       Includes :
c               momdim.inc
c               stdio.inc
c               rndoff.inc
c               eprmat.inc
c               eprdat.inc
c               indexf.inc
c               timer.inc
c
c       Uses :
c               matrxo.f
c               matrxd.f
c               csqz.f
c               cqrt.f
c               indexx.f
c
c*********************************************************************
c
      subroutine evesa(stv,eval,evec,bdw)
c
      implicit none
c
      include 'momdim.inc'
      include 'stdio.inc'
      include 'rndoff.inc'
      include 'eprdat.inc'
      include 'eprmat.inc'
      include 'indexf.inc'
      include 'timer.inc'
c
      integer i,j,k,m,n,ierr,jcol,kcol,indx
      integer bdw,bdwtmp,mxit
c
      double precision zr,zi,zero,wr,sqtol,tol
c
      complex*16 czero,cone,cmat,eval,evec,wt,cwsp,stv
      parameter (zero=0.D0,czero=(0.D0,0.D0),cone=(1.D0,0.D0))
      dimension cmat(xdim,xdim),eval(xdim),evec(xdim,xdim),wr(xdim),
     #          indx(xdim),stv(xdim,128),cwsp(xdim)
c
c#####################################################################
c
c----------------------------------------------------------------------
c     check to make sure that Exact Eigen System analysis desired
c----------------------------------------------------------------------
c
      if (itype.ne.4) then
        write (luttyo,5100) 
        write (lulog,5100)
        go to 999
      end if
c
c---------------------------------------------------------------------
c     calculate the matrix elements
c---------------------------------------------------------------------
c
      if (ips.ne.0) then
        call matrxo(ierr)
      else
        call matrxd(ierr)
      end if
c
      if (ierr.ne.0) then
        write (luttyo,1070) 
        write (lulog,1070)
        if (ierr.eq.1) then
          write (luttyo,1080)
          write (lulog,1080) 
        else if (ierr.eq.2) then
          write (luttyo,1090)
          write (lulog,1090)
        end if
        go to 999
      end if
c
c---------------------------------------------------------------------
c     convert the matrix into banded storage format
c---------------------------------------------------------------------
c
      do 105 j=1,xdim
      do 105 i=1,ndim
 105    cmat(i,j)=czero
c
      j=1
      k=1
c
      bdw=0
      do 110 m=1,ndim
        if ((abs(zdiag(1,m)).gt.rndoff).or.
     #      (abs(zdiag(2,m)).gt.rndoff)) then
          cmat(m,1)=dcmplx(zdiag(1,m),zdiag(2,m)) 
        endif
c
        if (jzmat(m).ne.jzmat(m+1)) then
          jcol=izmat(jzmat(m+1)-1)
        else
          jcol=m
        end if
c
        if (kzmat(m).ne.kzmat(m+1)) then
          kcol=izmat(mxel-kzmat(m+1)+2)
        else
          kcol=m
        end if
c
        bdwtmp=max(jcol,kcol)-m+1        
        if (bdwtmp.gt.bdw) bdw=bdwtmp
c
        do 120 n=m+1,max(jcol,kcol)
        if ((n.le.jcol).and.(n.eq.izmat(j))) then
          zi=zmat(j)
          j=j+1
        else
          zi=zero
        endif
        if ((n.le.kcol).and.(n.eq.izmat(mxel-k+1))) then
          zr=zmat(mxel-k+1)     
          k=k+1
        else
          zr=zero
        endif
c
        if ((abs(zr).gt.rndoff).or.(abs(zi).gt.rndoff)) then
          cmat(m,n-m+1)=dcmplx(zr,zi)
        endif
c
 120    continue
c
      if ((j.ne.jzmat(m+1)).or.(k.ne.kzmat(m+1))) then
        write (luttyo,2220)
c        stop
      endif
c
 110  continue
c
      if (bdw.gt.xdim) then
          write (luttyo,1200) bdw
          stop
      end if
c
c---------------------------------------------------------------------
c     convert the complex symmetric matrix into tridiagonal matrix
c---------------------------------------------------------------------
c
      sqtol=zero
c
      do 140 j=1,ndim
      do 140 i=1,ndim
 140    evec(i,j)=czero
c
      do 145 i=1,ndim
 145    evec(i,i)=cone
c
      call csqz(cmat,evec,xdim,xdim,xdim,xdim,ndim,bdw,sqtol)
c
c---------------------------------------------------------------------
c     Solve tridiagonal matrix for eigenvalues and eigenvectors
c---------------------------------------------------------------------
c
      mxit=100
c
      do 160 i=1,ndim
 160    cwsp(i)=czero
c
      call cqrt(cmat(1,1),cmat(1,2),evec,ndim,xdim,xdim,
     #          mxit,tol,cwsp,ierr)
c
      if (ierr.ne.0) then
        write (luttyo,2340) mxit
        call exit
      end if
c
c---------------------------------------------------------------------
c     calculate weighting of each eigenvector on the starting vector
c---------------------------------------------------------------------
c
      if (ips.ne.0) then
c				*** off-diagonal space
        do 300 i=1,ndim
          wt=czero
          do 301 j=1,ndim
 301        wt=wt+evec(j,i)*stv(j,1)
          wr(i)=cdabs(wt*wt)
 300    continue
      else
c				*** diagonal space
c     convert the eigenvectors of the diagonal space into the
c     corresponding off-diagonal space using the pulse propagator
c
        do 350 i=1,ndim
          k=1
          do 360 j=1,ndimo
            if (pid(j).eq.1) then
              evec(j,i)=-pp(k)*evec(k,i)
              k=k+1
            else
              evec(j,i)=-pp(k)*evec(k,i)-pp(k+1)*evec(k+1,i)
              k=k+2
            end if
 360      continue
c
          wr(i)=zero
          do 370 j=1,nstp
            wt=czero
            do 375 k=1,ndimo
 375          wt=wt+evec(k,i)*stv(k,j)
            wt=wt*wt
            if (cdabs(wt).gt.wr(i))  wr(i)=cdabs(wt)
 370      continue
 350    continue
c
      end if
c
c---------------------------------------------------------------------
c     order the eigenvalues and eigenvectors in weighting factors
c---------------------------------------------------------------------
c
      call indexx(ndim,wr,indx)
c
c	write (*,*) 'wtol = ',wtol
c        write (*,777) (i,wr(indx(i)),i=1,ndim)
c 777   format(2x,i5,4x,g14.7)
c
      nev=0
      do 400 i=1,ndim
        if (wr(indx(i)).lt.wtol) then
          nev=i-1
          go to 410
        end if
 400  continue
c
 410  do 420 i=1,nev
 420    eval(i)=cmat(indx(i),1)
c
      do 430 i=1,ndimo
        do 440 j=1,ndim
 440      cwsp(j)=evec(i,j)
        do 450 j=1,nev
 450      evec(i,j)=cwsp(indx(j))
 430  continue
c
 999  continue
c
      return
c
c=====================================================================
c     format statements
c=====================================================================
c
 1070 format(//,10x,
     #     '*** SERIOUS ERROR ENCOUNTERED IN MATRIX GENERATION ***')
 1080 format(/,20x,'*** TOO MANY OFF-DIAGONAL MATRIX ELEMENTS ***')
 1090 format(/,20x,'*** MATRIX DIMENSION TOO LARGE ***')
 1200 format(/,20X,'*** BANDWIDTH : ',i4,' IS TOO WIDE!! ***',/)
c
 2220 format(//,15x,'*** OOPS!, MATRIX FILE CORRUPTED ***',//)
 2340 format(/,5x,'!!! CQRT has not converged in',i4,' iterations !!!')
 5100 format(//,10x,
     #     '*** NOT EXACT EIGENSYSTEM ANALYSIS ***')
c
c=====================================================================
c
      end
