c**********************************************************************
c
c                       ==================
c                       SUBROUTINE : FBASO
c                       ==================
c
c       This routine builds a list of the basis set indices for
c       off-diagonal space in common block /indexf/ for the
c       truncation parameters set in /eprdat/.
c       If the file, <fileid>.indx exists, it simply reads the basis
c       set.  This basis would be the pruned basis obtained by running
c       eprbf and tnlf.  If the file does not exist, it generates
c       full basis set.
c
c       Modified by DEB 13-MAR-93 from program EPRBAS by DJS
c       Modified by Sanghyuk Lee 20-NOV-93
c
c       Includes:
c               stddim.inc
c               stdio.inc
c               eprdat.inc
c               indexf.inc
c
c       Uses:
c               ipar.f
c
c**********************************************************************
c
      subroutine fbaso(ixname)
c
      implicit none
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'indexf.inc'
c
      logical fexist
      character*30 ixname
      integer list,lu,lth
c
      integer lr,jkr,kr,krmx,jmr,mr,mrmx,iper,iqer,iqermn,iqermx,
     #     ipnr,ipnrmx,ipnrmn,iqnr,iqnrmx,iqnrmn,nrow,i
c
      integer ipar
      external ipar
c
c######################################################################
c
      inquire(file=ixname,exist=fexist)
      if (fexist) then
c
c----------------------------------------------------------------------
c     Read basis set information from index file
c----------------------------------------------------------------------
c
        open (unit=ludisk,file=ixname,status='old',
     #        access='sequential',form='unformatted')
        read (ludisk) ndim
        read (ludisk) (pi1(i),qi1(i),l1(i),jk1(i),k1(i),
     #                 jm1(i),m1(i),i=1,ndim)
        close (unit=ludisk)
c
        lemx=-1
        lomx=-1
        kmx=-1
        mmx=-3
        ipnmx=-3
c
        do 50 i=1,ndim
          if ((ipar(l1(i)).eq.1).and.(l1(i).gt.lemx)) lemx=l1(i)
          if ((ipar(l1(i)).ne.1).and.(l1(i).gt.lomx)) lomx=l1(i)
          if (k1(i).gt.kmx) kmx=k1(i)
          if (m1(i).gt.mmx) mmx=m1(i)
          if (pi1(i).gt.ipnmx) ipnmx=pi1(i)
 50     continue
        if (lomx.eq.-1) lomx=0
c
        go to 999
      end if
c
      nrow=0
c
c----------------------------------------------------------------------
c     *** loop over lr ***
c----------------------------------------------------------------------
c
      do 110 lr=0,lemx,ldelta
        if((ipar(lr).ne.1).and.(lr.gt.lomx)) go to 110
c
c----------------------------------------------------------------------
c     *** loop over jkr ***
c----------------------------------------------------------------------
c
        do 120 jkr=jkmn,1,2
c
c----------------------------------------------------------------------
c     *** loop over kr ***
c----------------------------------------------------------------------
c
        krmx=min(kmx,lr)
        do 130 kr=0,krmx,kdelta
          if((kr.eq.0).and.(ipar(lr).ne.jkr)) go to 130
c
c----------------------------------------------------------------------
c     *** loop over jmr ***
c----------------------------------------------------------------------
c
          do 140 jmr=jmmn,1,2
c
c----------------------------------------------------------------------
c     *** loop over mr ***
c----------------------------------------------------------------------
c
            mrmx=min(mmx,lr)
            do 150 mr=0,mrmx
c
c----------------------------------------------------------------------
c     *** loop over ipnr ***
c----------------------------------------------------------------------
c
              ipnrmx=min(in2,ipnmx)
              if (mr.eq.0) then
                ipnrmn=0
              else
                ipnrmn=-ipnrmx
              end if                
c
              do 160 ipnr=ipnrmn,ipnrmx
                if((mr.eq.0).and.(ipnr.eq.0).and.(ipar(lr).ne.jmr))
     #                                 go to 160
                if((ipsi0.eq.0).and.(ipnr.ne.mr)) go to 160
c
c----------------------------------------------------------------------
c     *** loop over iqnr ***
c----------------------------------------------------------------------
c
                iqnrmx=in2-iabs(ipnr)
                iqnrmn=-iqnrmx
                do 170 iqnr=iqnrmn,iqnrmx,2
c
                  nrow=nrow+1
                  if (nrow.le.mxdim) then
                    jqe1(nrow)=0
                    l1(nrow)=lr
                    jk1(nrow)=jkr
                    k1(nrow)=kr
                    jm1(nrow)=jmr
                    m1(nrow)=mr
                    pi1(nrow)=ipnr
                    qi1(nrow)=iqnr
                  end if
c
c----------------------------------------------------------------------
c     end loop over rows
c----------------------------------------------------------------------
c
 170              continue
 160            continue
 150          continue
 140        continue
 130      continue
 120    continue
 110  continue
c
      ndim=nrow
c
c----------------------------------------------------------------------
c     Write basis set information from index file
c----------------------------------------------------------------------
c
      if ((itype.ne.2).or.(nort.le.1)) then
c
        ptol=0.0D0
c
        open (unit=ludisk,file=ixname,status='new',
     #        access='sequential',form='unformatted')
        write (ludisk) ndim
        write (ludisk) (pi1(i),qi1(i),l1(i),jk1(i),k1(i),
     #                 jm1(i),m1(i),i=1,ndim)
        close (unit=ludisk)
      end if
c
 999  ndimo=ndim
      return
      end
