c----------------------------------------------------------------------
c                    =====================
c                     subroutine SETNAM
c                    =====================
c
c Given a generic name in <fileid>, this subroutine removes any
c file extension from <fileid> and determines the names for the
c various possible output files associated with EPRLFX or EPRLFZ
c slow-motional calculations by appending a file extension
c in the form '.xxx?'. The extensions are
c
c      prname : <fileid>.PAR?   Parameter file (unformatted)
c      fmname : <fileid>.FMT?   Parameter listing (formatted)
c      bsname : <fileid>.BSS?   Basis set indices
c      spname : <fileid>.SPC?   Calculated spectrum
c      rlname : <fileid>.RLF?   Data from swept-field CG calculation
c      tdname : <fileid>.TDL?   Tridiagonal SLE matrix (unformatted)
c      tfname : <fileid>.TDF?   Tridiagonal SLE matrix (formatted)
c      mtname : <fileid>.MTX?   SLE matrix (unformatted)
c      mfname : <fileid>.MTF?   SLE matrix (formatted)
c      ixname : <fileid>.INDX   basis set index list for pS=1 (unformatted)
c      vtname : <fileid>.STV?   Starting vector
c      tsname : <fileid>.MTS?   MTS database file
c      egname : <fileid>.EGV?   SLE eigenvalues/weights
c      evname : <fileid>.EVC?   SLE eigenvectors
c      efname : <fileid>.EFV?   SLE eigenvectors (formatted)
c      ername : <fileid>.ERR?   Error estimates in CG procedure
c      dename : <fileid>.DEV?   Distinctive eigenvalues
c
c ? specifies each pS subspace:
c             ?=x    <---> pS=+1 subspace
c             ?=y    <---> pS=-1 subspace
c             ?=z    <---> pS=0  subspace
c
c             ?=o    <---> backup files for storing the unpruned basis set
c
c  Files with ?=v are necessary for the calculation of the t1-dependent
c  starting vector
c
c  Includes
c     stddim.inc   Definition of mxcalc
c     fnames.inc   Definition of names in common /fnames/
c     eprdat.inc   Definition of ips
c
c  NB: Although the code in namdef.inc and this routine are standard
c      FORTRAN 77, they may not be portable to installations that
c      do not fully implement the standard.
c
c----------------------------------------------------------------------
      subroutine setnam
c
      implicit none
c
      include 'stddim.inc'
      include 'fnames.inc'
      include 'eprdat.inc'
c
      integer idot,iend,whris
      external whris
c
      iend = whris( fileid, ' ' )-1
c
      if(ips.eq.1) then
         prname=fileid(:iend)//'.parx'
         fmname=fileid(:iend)//'.fmtx'
         bsname=fileid(:iend)//'.bssx'
         spname=fileid(:iend)//'.spcx'
         rlname=fileid(:iend)//'.rlfx'
         tdname=fileid(:iend)//'.tdlx'
         tfname=fileid(:iend)//'.tdfx'
         mtname=fileid(:iend)//'.mtxx'
         mfname=fileid(:iend)//'.mtfx'
         ixname=fileid(:iend)//'.indx'
         vtname=fileid(:iend)//'.stvx'
         tsname=fileid(:iend)//'.mtsx'
         egname=fileid(:iend)//'.egvx'
         evname=fileid(:iend)//'.evcx'
         efname=fileid(:iend)//'.efvx'
         ername=fileid(:iend)//'.errx'
         dename=fileid(:iend)//'.devx'
         prnamo=fileid(:iend)//'.paro'
         ixnamo=fileid(:iend)//'.indo'
      else if(ips.eq.-1) then
         prname=fileid(:iend)//'.pary'
         fmname=fileid(:iend)//'.fmty'
         bsname=fileid(:iend)//'.bssy'
         spname=fileid(:iend)//'.spcy'
         rlname=fileid(:iend)//'.rlfy'
         tdname=fileid(:iend)//'.tdly'
         tfname=fileid(:iend)//'.tdfy'
         mtname=fileid(:iend)//'.mtxy'
         mfname=fileid(:iend)//'.mtfy'
         ixname=fileid(:iend)//'.indx'
         vtname=fileid(:iend)//'.stvy'
         tsname=fileid(:iend)//'.mtsy'
         egname=fileid(:iend)//'.egvy'
         evname=fileid(:iend)//'.evcy'
         efname=fileid(:iend)//'.efvy'
         ername=fileid(:iend)//'.erry'
         dename=fileid(:iend)//'.devy'
      else if(ips.eq.0) then
         prname=fileid(:iend)//'.parz'
         fmname=fileid(:iend)//'.fmtz'
         bsname=fileid(:iend)//'.bssz'
         spname=fileid(:iend)//'.spcz'
         rlname=fileid(:iend)//'.rlfz'
         tdname=fileid(:iend)//'.tdlz'
         tfname=fileid(:iend)//'.tdfz'
         mtname=fileid(:iend)//'.mtxz'
         mfname=fileid(:iend)//'.mtfz'
         ixname=fileid(:iend)//'.indx'
         vtname=fileid(:iend)//'.stvz'
         tsname=fileid(:iend)//'.mtsz'
         egname=fileid(:iend)//'.egvz'
         evname=fileid(:iend)//'.evcz'
         efname=fileid(:iend)//'.efvz'
         ername=fileid(:iend)//'.errz'
         dename=fileid(:iend)//'.devz'
c
c   following files are needed in the calculation of t1-dependent
c   starting vector
c
         egnamv=fileid(:iend)//'.egvx'
         evnamv=fileid(:iend)//'.evcx'
         vtnamv=fileid(:iend)//'.stvx'
      else
         nameOK=.false.
      end if
      nameOK=.true.
      namlth=iend+5
      return
      end
