c**********************************************************************
c                    =========================
c                       subroutine STVECP
c                    =========================
c
c       subroutine for calculating the starting vector.
c       Rewritten by Sanghyuk Lee from routine STVECF by David Budil
c       to make the starting vector as the equilibrium z-magnetization
c       
c       Notes:
c               1) The orientational integral is evaluated numerically 
c                  using the Curtis-Clenshaw-Romberg extrapolation algorithm.
c                  For details, see Bruno's thesis.
c
c       Includes:
c               stddim.inc
c               eprdat.inc
c               rndoff.inc
c               indexf.inc
c
c       Uses:
c               ipar.f
c               ccrint.f
c
c**********************************************************************
c
      subroutine stvecp(v)
c
      include 'stddim.inc'
      include 'eprdat.inc'
      include 'rndoff.inc'
      include 'indexf.inc'
c
      logical flr,fkr
c
      integer i,iparlr,nup,id,nrow,ipar,
     #        iper,jqer,lr,jkr,kr,jmr,mr,ipnr,iqnr
c
      double precision cnl,stvt,stvec,factor,dsq2,vnorm
c
      double precision acc,sml,fz,one,zero,two
      parameter (acc=1.0D-8,sml=1.0D-10)
      parameter (one=1.0D0,zero=0.0D0,two=2.0D0)
c
      double precision frac,efrac,nfrac
      parameter (efrac=one,nfrac=two/3.0D0)
c
      common /ifzdat/lr,kr
c
      double precision v(2,1)
c
      external fz,ipar
c
c######################################################################
c
      dsq2=sqrt(2.0D0)
c
c----------------------------------------------------------------------
c             initialize counters
c----------------------------------------------------------------------
c
      vnorm=zero
      frac=one
c
      iper=ips
c
c----------------------------------------------------------------------
c             *** loop over rows ***
c----------------------------------------------------------------------
c
      do 100 nrow=1,ndim
c
        v(1,nrow)=zero
c
        jqer=jqe1(nrow)
        lr=l1(nrow)
        jkr=jk1(nrow)
        kr=k1(nrow)
        jmr=jm1(nrow)
        mr=m1(nrow)
        ipnr=pi1(nrow)
        iqnr=qi1(nrow)
c
        if ( ((ips.ne.0).and.(jqer.ne.0)) .or. 
     #       ((ips.eq.0).and.(jqer.ne.1)) ) go to 100
        if ( (jkr.ne.1).or.(jmr.ne.1).or.
     #      (mr.ne.0).or.(ipnr.ne.0) ) go to 100
c
        iparlr=ipar(lr)
        flr=iparlr.eq.1
        cnl=zero
        if(flr) cnl=dsqrt(dble(2*lr+1))
c
        fkr=ipar(kr).eq.1
        stvt=zero
        if(flr.and.fkr) then
          if(lptmx.eq.0) then
            if((lr.eq.0).and.(kr.eq.0)) stvt=one
          else if((kptmx.eq.0).and.(kr.ne.0)) then
            stvt=zero
          else
            call ccrint(zero,one,acc,sml,stvt,nup,fz,id)
            if(kr.ne.0) then
              factor=one
              do 500 i=lr-kr+1,lr+kr
                factor=factor*dble(i)
 500          continue
              factor=one/dsqrt(factor)
            else
              factor=one/dsq2
            end if
            stvt=factor*stvt*cnl
          end if
        end if
c
c      **  starting vector for nitroxide  **
c
        if(iper.eq.0) then
          if(iqnr.eq.2) frac=one
          if(iqnr.eq.0) frac=two
          if(iqnr.eq.-2) frac=3.5D0
        end if
c
c      **  center of loops  **
c
        stvec=stvt*frac
        v(1,nrow)=stvec
        vnorm=vnorm+stvec*stvec
c
 100  continue
c
c----------------------------------------------------------------------
c     normalize starting vector and zero out imaginary part
c----------------------------------------------------------------------
c
      nelv=0
      vnorm=one/dsqrt(vnorm)
      do 200 i=1,ndim
        v(1,i)=v(1,i)*vnorm
        if(abs(v(1,i)).gt.rndoff) then
          nelv=nelv+1
        else
          v(1,i)=zero
        end if
        v(2,i)=zero
 200  continue
c
c----------------------------------------------------------------------
c     zero out remainder of vector
c----------------------------------------------------------------------
c
c      do 210 i=ndim+1,mxdim
c        v(1,i)=zero
c        v(2,i)=zero
c 210  continue        
c
c----------------------------------------------------------------------
c     return to caller
c----------------------------------------------------------------------
c
      return
      end
