c*********************************************************************
c
c                       ===============
c                       PROGRAM : STVPD
c                       ===============
c
c       This program calculates the projection of each eigenvector on
c       the time-dependent density matrix vector in diagonal subspace.
c       This can be a good criterion of choosing the significant eigen-
c       modes in the evolution of the diagonal subspace, considering
c       the starting vector in the diagonal subspace was somewhat
c       arbitrary.
c
c       Includes:
c               stdio.inc
c               eprdat.inc
c               tdexp.inc
c               tdfnam.inc
c
c       Uses:
c               getidt.f
c               tdsnam.f
c               rdfdat.f
c               rdfinp.f
c
c*********************************************************************
c
      program stvpd
c
      implicit none
c
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'tdexp.inc'
      include 'tdfnam.inc'
c
      logical fexist    
      integer i,j,k,n,mxcal,mxt1,ncalc,it1,it2,ierr
      double precision t1,t2,mxprob,tmp,pptol
c
      complex*16 czero,ci,wt,sum,prbsum
c
      double precision zero,one,two,mone,pi,twopi,gtf,cfact
      parameter (zero=0.0D0,one=1.0D0,two=2.0D0,mone=1.0D-3)
      parameter (pi=3.1415926535897932384D0,gtf=2.802471512)
      parameter (czero=(0.0D0,0.0D0),ci=(0.0D0,1.0D0))
c
      integer mxdim0,mxdim1,mxegv1,mxegv0,mxpts
      parameter (mxdim0=2000,mxdim1=1200,mxegv1=1200,mxegv0=2000,
     #        mxpts=256)
      complex*16 egvalx,egvalz,stv,oomat,odmat,otstv,plsm,prjs,
     #        sumprjs
      integer indx(mxdim0)
      double precision mxpb(mxdim0)
      common /tdmat/ egvalx(mxdim1),egvalz(mxdim0),stv(mxdim1),
     #    oomat(mxdim1,mxegv1),odmat(mxdim1,mxegv0),otstv(mxegv1),
     #    plsm(mxegv1,mxegv0),prjs(mxdim1,mxpts),sumprjs(mxpts)
c
c
c#####################################################################
c
c----------------------------------------------------------------------
c Timing call changed for compatibility with UNIX
c      call cputime(cpu0,ircode)
c----------------------------------------------------------------------
c
      twopi=two*pi
      cfact=twopi*gtf
c
      write (luttyo,1000)
      write (luttyo,1010)
      write (luttyo,1020)
      read (luttyi,*) pptol
c
      mxcal=10
      call getidt(n,mxcal)
      ioflid = iofile(1)
      dtflid = dtfile(1)
      call tdsnam
c
      open (unit=lulog,file=prjnam,access='sequential',
     #      form='formatted')
c
      rewind (unit=lulog)
c
c----------------------------------------------------------------------
c     check for existence and open parameter file
c----------------------------------------------------------------------
c
      inquire(file=ipname,exist=fexist)
      if (fexist) then
        call rdfinp(ipname)
      else
        write (luttyo,1100) ipname(:namlth)
        go to 999
      end if
c
      inquire(file=prnamz,exist=fexist)
      if (fexist) then
        call rdfdat(prnamz,ierr)
      else
        write (luttyo,1100) prnamz(:namlth)
        go to 999
      end if
c
      field=b0
      ndim1=ndimo
      nev1=nevo
      nev0=nev
c
c----------------------------------------------------------------------
c     read the files that contains starting vector, eigenvalues and
c     eigenvectors for pS=1 subspace and for pS=0 subspace if necessary.
c----------------------------------------------------------------------
c
c                    *** read the starting vector : pS=1 ***
c
      write (luttyo,2000)
      open (unit=ludisk,file=vtnamx,status='old',
     #     access='sequential',form='unformatted')
      read (ludisk) (stv(i),i=1,ndim1)
      close (unit=ludisk)
c                    *** read the eigenvalues : pS=1 ***
c
      open (unit=ludisk,file=egnamx,status='old',
     #     access='sequential',form='formatted')
      read (ludisk,2200) (j,k,egvalx(i),wt,i=1,nev1)
      close (unit=ludisk)
c  substract b0 term from the eigenvalues
      do 15 i=1,nev1
 15     egvalx(i)=egvalx(i)+ci*field
c
c                    *** read the eigenvectors : pS=1 ***
c
      open (unit=ludisk,file=evnamx,status='old',
     #      access='sequential',form='unformatted')
      do 20 j=1,nev1
 20     read (ludisk) (oomat(i,j),i=1,ndim1)
      close (unit=ludisk)
c
c                    *** read the eigenvalues : pS=0 ***
c
      inquire(file=egnamo,exist=fexist)
      if (fexist) then
        open (unit=ludisk,file=egnamo,status='old',
     #       access='sequential',form='formatted')
        read (ludisk,2200) (indx(i),k,egvalz(i),wt,i=1,nev0)
        close (unit=ludisk)
      else
        write (luttyo,2100)
        write (lulog,2100)
        go to 999
      end if
c                    *** read the eigenvectors : pS=0 ***
c                    Note that the eigenvectors are already converted
c                    into the pS=-1 space.
      inquire(file=evnamo,exist=fexist)
      if (fexist) then
        open (unit=ludisk,file=evnamo,status='old',
     #        access='sequential',form='unformatted')
        do 40 j=1,nev0
 40       read (ludisk) (odmat(i,j),i=1,ndim1)
        close (unit=ludisk)
      else
        write (luttyo,2100)
        write (lulog,2100)
        go to 999
      end if
c
c---------------------------------------------------------------------
c     Conversion of unit of time constants and the eigenvalues
c---------------------------------------------------------------------
c
      tfix=tfix*mone
      if (idtf.eq.0) then
        init1=init1*mone
        stpt1=stpt1*mone
        init2=init2*mone
        stpt2=stpt2*mone
      else
        write (luttyo,2300)
        write (lulog,2300)
        go to 999
      end if
c
      do 60 i=1,nev1
 60     egvalx(i)=egvalx(i)*cfact
c
      do 100 i=1,nev1
        sum=czero
        do 110 j=1,ndim1
 110      sum=sum+oomat(j,i)*stv(j)
 100    otstv(i)=sum
c
      do 200 i=1,nev1
      do 200 j=1,nev0
        sum=czero
        do 205 k=1,ndim1
 205      sum=sum-oomat(k,i)*odmat(k,j)
 200    plsm(i,j)=sum
c
c---------------------------------------------------------------------
c     calculate the probability
c---------------------------------------------------------------------
c
      write (luttyo,2500)
c
      t1=zero
      do 300 it1=1,npt1
        t1=init1+(it1-1)*stpt1
        do 310 i=1,nev0
          sum=czero
          do 320 j=1,nev1
 320        sum=sum+plsm(j,i)*cdexp(-egvalx(j)*t1)*otstv(j)
          prjs(i,it1)=sum*sum
 310    continue
 300  continue
c
      write (lulog,3000)
      do 340 it1=1,npt1
        prbsum=czero
        do 345 i=1,nev0
 345      prbsum=prbsum+prjs(i,it1)
        sumprjs(it1)=prbsum
        write (lulog,3010) it1,cdabs(prbsum)
 340  continue
c
      write (lulog,3100)
      do 360 i=1,nev0
        mxt1=0
        mxprob=zero
        do 365 it1=1,npt1
          tmp=cdabs(prjs(i,it1))
          if (tmp.gt.mxprob) then
            mxprob=tmp
            mxt1=it1
          end if
 365    continue
        write (lulog,3110) i,mxt1,mxprob
        mxpb(i)=mxprob
 360  continue
c
      k=0
      wt=czero
c
      open (unit=ludisk,file=egnamz,status='unknown',
     #     access='sequential',form='formatted')
      do 400 i=1,nev0
        if (mxpb(i).gt.pptol) then
          k=k+1
          write (ludisk,2210) k,indx(i),egvalz(i),wt
        end if
 400  continue
      close (unit=ludisk)
c
      open (unit=ludisk,file=evnamz,status='unknown',
     #      access='sequential',form='unformatted')
      do 410 j=1,nev0
        if (mxpb(j).gt.pptol) write (ludisk) (odmat(i,j),i=1,ndim1)
 410  continue
c
      close (unit=ludisk)
c
      write (luttyo,4000) nev0,k,pptol
      write (lulog,4000) nev0,k,pptol
c
c      do 380 i=1,nev0
c        write (luttyo,3200) i
c        do 385 it1=1,npt1
c 385      write (luttyo,3210) it1,cdabs(prjs(i,it1))
c 380  continue
c
 999  close (unit=lulog)
c
      stop
c
c=====================================================================
c     format statements
c=====================================================================

c
 1000 format(//,2x,70('#'),//)
 1010 format(25x,'PROGRAM : STVPD',/,25x,'---------------',/)
 1020 format(2x,'Enter the tolerance for the probability : ')
 1100 format(/,20x,'*** CANNOT FIND FILE ',a,' ***',/)
c
 2000 format(/,8x,'*** Reading Starting vector, eigenvalues and',
     #       ' eigenvectors ***')
 2100 format(/,10x,'*** move <id>.egvz & <id>.evcz to <id>.egvo & ',
     #       '<id>.evco ***')
 2200 format(i4,2x,i4,4x,g14.7,1x,g14.7,2x,g14.7,1x,g14.7)
 2210 format(i4,2x,i4,4x,g14.7,',',g14.7,2x,g14.7,',',g14.7)
 2300 format(/,10x,'*** Time domain calculation is required!! ***')
c
 2500 format(/,18x,'*** Calculating probability ***',/)
c
 3000 format(//,'##',2x,'t1 No.',4x,'sum of prob.',2x,'##')
 3010 format(5x,i3,9x,g10.4)
 3100 format(//,'##',2x,'egval No.',4x,'t1 mx No.',5x,'mx prob.',
     #       2x,'##')
 3110 format(6x,i4,8x,i3,9x,g10.4)
 3200 format(//,'##',2x,'egval No. : ',i4,2x,'##',/,2x,'t1 No.',5x,
     #       'probability')
 3210 format(2x,i3,5x,g10.4)
c
 4000 format(/,2x,i4,' eigenvalues stored with pptol = 0.0',
     #       /,2x,i4,' eigenvalues stored with pptol = ',g10.5)
c=====================================================================
c
      end
