c**********************************************************************
c
c                       ==============
c                       PROGRAM : TNLF
c                       ==============
c
c       This program reads in the basis set truncation file 
c       written by eprbf and lists the data on the screen.
c
c       Includes:
c               stddim.inc
c               stdio.inc
c               eprdat.inc
c               indexf.inc
c               rndoff.inc
c
c       Uses:
c               getids.f
c               setnam.f
c               rdfdat.f
c               fbaso.f
c               ipar.f
c
c       written by DJS 11-SEP-87
c       modified by DJS 27-MAY-88 discard generation of input data file
c                                 for EXPBAS.PRO
c       rewritten for EPRLF family programs DEB
c
c**********************************************************************
c
      program tnlf
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'indexf.inc'
      include 'fnames.inc'
      include 'rndoff.inc'
c
      integer i,j,maxle,maxlo,maxk,maxm,maxpi,maxlkm,
     #     minle,minlo,mink,minm,minpi,n,nfiles
      double precision basis(2,mxdim),bmax,bcut
      logical fexist
c
c######################################################################
c
      write(luttyo,1000)
      write(luttyo,1010)
c
      call getids(nfiles,1)
      if (nfiles.eq.0) stop
      fileid=files(1)
c
      ips=idxps(1)
      call setnam
c
c----------------------------------------------------------------------
c     read in parameters from parameter file
c----------------------------------------------------------------------
c
	inquire(file=prname,exist=fexist)
c
	if (fexist) then 
          call rdfdat(prname,ierr)
        else
	  write (luttyo,1100) prname(:namlth)
          stop
        endif
c
      open (unit=ludisk,file=bsname,status='old',
     #     access='sequential',form='unformatted')
      read (ludisk) (basis(1,i),basis(2,i),i=1,ndim)
      close (unit=ludisk)
c
c----------------------------------------------------------------------
c     write out header for database file
c----------------------------------------------------------------------
c
      open (unit=ludisk1,file=tsname,status='unknown',
     #     access='sequential',form='formatted')
c
      write(ludisk1,1030) in2,ipdf,ist,ipt
      write(ludisk1,1030) lemx,lomx,jkmn,kmx,mmx,ipnmx
      write(ludisk1,1030) ldelta,kdelta,nstep,ndim,neltot,nfield,itype
      write(ludisk1,1030) itm,itd,ipsi0,lkmtrc,lkmdff,jkmn
      write(ludisk1,1050) ((fgd(i,j),j=1,5),i=1,2)
      write(ludisk1,1050) ((fad(i,j),j=1,5),i=1,2)
      write(ludisk1,1060) gxx,gyy,gzz,axx,ayy,azz
      write(ludisk1,1060) g0,a0,b0,pl,pkxy,pkzz
      write(ludisk1,1060) dxy,dzz,tl,tkxy,tkzz,djf
      write(ludisk1,1060) psi,bed,gad,alm,bem,gam
      write(ludisk1,1060) gamman,oss,cgtol,fieldi,fieldf
      write(ludisk1,1050) cpot
c
      write(luttyo,1080)
      write(ludisk1,1080)
c
c----------------------------------------------------------------------
c     normalize to the largest value
c----------------------------------------------------------------------
c
      bmax=basis(2,1)
      do 10 i=2,ndim
        if (bmax.lt.basis(2,i)) bmax=basis(2,i)
 10   continue
c
      do 20 i=1,ndim
        basis(2,i)=basis(2,i)/bmax
 20   continue
c
c----------------------------------------------------------------------
c     Generate a list of basis elements
c----------------------------------------------------------------------
c
      call fbaso(ixname)
c
c----------------------------------------------------------------------
c     loop over cutoff values
c----------------------------------------------------------------------
c                                      
      do 200 i=1,16
c
        maxle=-1
        minle=lemx+1
        maxlo=-1
        minlo=lomx+1
        maxk=-1
        mink=kmx+1
        maxm=-(mmx+1)
        minm=-maxm
        maxpi=-(ipnmx+1)
        minpi=ipnmx+1
        maxlkm=-1
c
        bcut=0.5D0**i
c
c----------------------------------------------------------------------
c     loop over basis set data
c----------------------------------------------------------------------
c                                      
        do 210 j=1,ndim
c
          if (basis(2,j).lt.bcut) go to 210
c
          n=l1(j)
          if (ipar(n).eq.1) then
            if (n.gt.maxle) maxle=n
            if (n.lt.minle) minle=n
          else
            if (n.gt.maxlo) maxlo=n
            if (n.lt.minlo) minlo=n
          end if
c
          n=k1(j)
          if (n.gt.maxk) maxk=n
          if (n.lt.mink) mink=n
c
          n=m1(j)
          if (n.gt.maxm) maxm=n
          if (n.lt.minm) minm=n
c
          n=pi1(j)
          if (n.gt.maxpi) maxpi=n
          if (n.lt.minpi) minpi=n
c
          n=l1(j)+k1(j)+iabs(m1(j))-lemx
          if (n.gt.maxlkm) maxlkm=n
c
 210    continue
c
        write(ludisk1,1090) bcut,minle,maxle,minlo,maxlo,mink,maxk,
     #       minm,maxm,minpi,maxpi,maxlkm
c
        write(luttyo,1090) bcut,minle,maxle,minlo,maxlo,mink,maxk,
     #       minm,maxm,minpi,maxpi,maxlkm
c
 200  continue
c
c----------------------------------------------------------------------
c     exit from program
c----------------------------------------------------------------------
c
      close(unit=ludisk1)
      write(luttyo,1000)
      stop
c
c======================================================================
c    format statements
c======================================================================
c
 1000 format(//,2x,70('#'),//)
 1010 format(28x,'PROGRAM : TNLF',/,28x,'--------------',/)
 1030 format(7(1x,i6))
 1050 format(5(1x,g12.5))
 1060 format(6(1x,g12.5))
 1080 format(//2x,70('-')/
     #       3x,'error',7x,'Le',8x,'Lo',8x,'K',9x,'M',9x,'pI',
     #       5x,'LKMdff'/,12x,5('min  max  '),'max'/ )
 1090 format(g10.3,11(i5))
 1100 format(/,20x,'*** CANNOT FIND FILE ',a,' ***',/)
c
c======================================================================
c
      end
