c  VERSION 1.0  (NLSP version)   4/26/93
c----------------------------------------------------------------------
c                    =========================
c                       subroutine ADDPRM
c                    =========================
c
c Add a parameter to list of parameters being varied for nonlinear
c least-squares. Also maintain the lists of 
c    (1) ibnd    : boundary flag for variable parameter
c    (2) prmin   : minimum for variable parameter
c    (3) prmax   : maximum for variable parameter
c    (4) prscl   : desired accuracy for given parameter
c    (5) xfdstp  : Size of forward-differences step
c    (6) tag     : Name of the parameter (character*6)
c    (7) ixx     : index of each element in the fparm array into the x vector
c
c The values of these parameters for the given variable parameter
c are passed to ADDPRM.
c
c Each addition to the list is included in sort order with respect to
c the index ix. This is so that, if the parameter gib is being varied,
c they will be varied before the other parameters.
c
c----------------------------------------------------------------------
      subroutine addprm(ix,ibd,prmn,prmx,prsc,step,ident)
      implicit none
      integer ix,ibd
      double precision prmn,prmx,prsc,step
      character*6 ident
c
      include 'nlsdim.inc'
      include 'eprprm.inc'
      include 'expdat.inc'
      include 'parcom.inc'
      include 'lpnam.inc'
      include 'lmcom.inc'
      include 'stdio.inc'
      include 'rndoff.inc'
c
      integer i,ixabs,j,k,k1,nmov
c
      logical tcheck
      external tcheck
c
c######################################################################
c
c----------------------------------------------------------------------
c See if there is enough room in the parameter list
c----------------------------------------------------------------------
      if (nprm+1.gt.mxvar) then
         write(luout,1001) mxvar
         if (luout.ne.luttyo) write(luttyo,1001) mxvar
         return
      end if
c
      if (.not.tcheck(ix,ident,luttyo)) return
c
c----------------------------------------------------------------------
c Search parameter list to see if parameter is already there
c----------------------------------------------------------------------
      ixabs=abs(mod(ix,100))
      do 10 i=1,nprm
c                                      *** parameter already in list
         if (ixpr(i).eq.ixabs) then
            write(luout,1000) ident
            if (luout.ne.luttyo) write(luttyo,1000) ident
            return
         else if (ixpr(i).gt.ixabs) then
            go to 14
         end if
 10   continue
c
c----------------------------------------------------------------------
c Reached end of list: parameter may be added to the end
c----------------------------------------------------------------------
      i=nprm+1
      nprm=nprm+1
      go to 18
c
c----------------------------------------------------------------------
c Found proper location for new parameter: move top of list up
c----------------------------------------------------------------------
 14   nmov=nprm-i+1
      do 16 j=1,nmov
         k=nprm-j+1
         k1=k+1
         x(k1)=x(k)
         ixpr(k1)=ixpr(k)
         prmin(k1)=prmin(k)
         prmax(k1)=prmax(k)
         prscl(k1)=prscl(k)
         xfdstp(k1)=xfdstp(k)
         ibnd(k1)=ibnd(k)
         tag(k1)=tag(k)
         ixx(ixpr(k1))=ixx(ixpr(k1))+1
 16   continue
      nprm=nprm+1
c
c----------------------------------------------------------------------
c  Add new parameter(s) to proper location list
c----------------------------------------------------------------------
 18   x(i)=fparm(ixabs)
      ixx(ixabs)=i
      ixpr(i)=ixabs
      prmin(i)=prmn
      prmax(i)=prmx
      prscl(i)=prsc
      xfdstp(i)=step
      if(dabs(xfdstp(i)).lt.rndoff) xfdstp(i)=step
      ibnd(i)=ibd
      tag(i)=ident
c
      write(luout,1003) nprm
      if (luout.ne.luttyo) write(luttyo,1003) nprm
      return
c
c #### format statements ###########################################
c
 1000 format('*** Parameter ''',a,''' is already being varied ***')
 1001 format('*** Limit of',i2,' variable parameters exceeded ***')
 1003 format(i3,' parameters now being varied')
      end
