c NLSP VERSION (VERSION 1.0) 7/5/93
c----------------------------------------------------------------------
c                         ====================
c                          subroutine CONVTC
c                         ====================
c
c     Subroutine to convert the tensor named on the passed command line
c     according to the symmetry specified by iflg. The following are
c     valid symmetries: 
c         1:  Cartesian
c         2:  Spherical
c         3:  Axial
c     See source file tensym.f for an explanation of these symmetries.
c
c     Uses:
c         gettkn, touppr (in strutl.f)
c         ipfind.f
c         rmvprm.f
c         tocart, tosphr, toaxil (in tensym.f)
c
c     Includes 
c         nlsdim.inc
c         eprprm.inc
c         parcom.inc
c         stdio.inc
c         lpnam.inc
c
c---------------------------------------------------------------------- 

      subroutine convtc( line,iflg )
      implicit none
      character line*80, token*30, ident*6
      integer i,iflg,ix,ixa,ixf,ixten,j,k,lth,lu
      integer CARTESIAN,SPHERICAL,AXIAL
      parameter (CARTESIAN=1,SPHERICAL=2,AXIAL=3)
c
      include 'nlsdim.inc'
      include 'eprprm.inc'
      include 'parcom.inc'
      include 'stdio.inc'
      include 'lpnam.inc'
c
      integer ipfind
      external ipfind
c
c######################################################################
c
      if (iflg.lt.CARTESIAN .or. iflg.gt.AXIAL) iflg=CARTESIAN
c
      call gettkn(line,token,lth)
      call touppr(token,lth)
c
c                                          *** No tensor specified
      if (lth.eq.0) then
         write (luttyo,1000)
         return
      end if
c
      lth=1
      ix=ipfind(token,lth)
      ixa=abs(mod(ix,100))
      if (ix.gt.100) ixa=0
c     
      if (ixa.eq.0 .or. ixa-IGXX.ge.nalias) then
c
c                                          *** Unknown tensor
         write (luout,1001) token(1:1)
         if (luout.ne.luttyo) write (luttyo,1001) token(1:1)
         return
c
c----------------------------------------------------------------------
c     Tensor found: Check existing symmetry
c----------------------------------------------------------------------
      else
         ixf=IIGFLG+(ixa-IGXX)/3
         ixten=IGXX+3*(ixf-IIGFLG)
c
c----------------------------------------------------------------------
c        Symmetry not set yet: set it and exit
c----------------------------------------------------------------------
         if (iparm(ixf) .eq. 0) then
            iparm(ixf)=iflg
            write(luttyo,1002) token(1:1),symstr(iflg)
            return
c
c----------------------------------------------------------------------
c        Symmetry same as the one specified: exit
c----------------------------------------------------------------------
         else if (iparm(ixf).eq.iflg) then
            return
         end if
c
c----------------------------------------------------------------------
c        Remove any tensor components of the old symmetry from
c        the list of variable parameters
c----------------------------------------------------------------------
         do 10 i=0,2
            j=ixten+i
            if (iparm(ixf).eq.CARTESIAN) then
               k=j
            else if (iparm(ixf).eq.SPHERICAL) then
               k=-j
            else if (iparm(ixf).eq.AXIAL) then
               k=-(100+j)
            end if
            ident=tag(ixx(j))
            if (ixx(j).ne.0) call rmvprm(k,ident)
 10      continue
c     
c----------------------------------------------------------------------
c        Now...convert the tensor symmetry!
c----------------------------------------------------------------------
         if (iflg.eq.2) then
            call tosphr( fparm(ixten),iparm(ixf) )
         else if (iflg.eq.3) then
            call toaxil( fparm(ixten),iparm(ixf) )
         else
            call tocart( fparm(ixten),iparm(ixf) )
         end if
         iparm(ixf)=iflg
      end if
      write (luout,1003) token(1:1),symstr(iflg)
      if (luout.ne.luttyo) write (luttyo,1003) token(1:1),symstr(iflg)
c     
c----------------------------------------------------------------------
c        Report new values
c----------------------------------------------------------------------
      lu=luout
 300  if (ixf.eq.IIGFLG) then
c                                                  * g tensor
         if (iflg.eq.1) then
            write (lu,1011) (fparm(ixten+i),i=0,2)
         else if (iflg.eq.2) then
            write (lu,1012) (fparm(ixten+i),i=0,2)
         else
            write (lu,1013) (fparm(ixten+i),i=0,2)
         end if
      else if (ixf.eq.IIGFLG+1) then
c                                                  * hf tensor
         if (iflg.eq.1) then
            write (lu,1021) (fparm(ixten+i),i=0,2)
         else if (iflg.eq.2) then
            write (lu,1022) (fparm(ixten+i),i=0,2)
         else
            write (lu,1023) (fparm(ixten+i),i=0,2)
         end if
      else
c                                                  * r tensor
         if (iflg.eq.1) then
            write (lu,1031) (fparm(ixten+i),i=0,2)
         else if (iflg.eq.2) then
            write (lu,1032) (fparm(ixten+i),i=0,2)
         else
            write (lu,1033) (fparm(ixten+i),i=0,2)
         end if
      end if
      if (lu.ne.luttyo) then
         lu=luttyo
         go to 300
      end if
c
      return
c     
 1000 format('*** No tensor specified ***')
 1001 format('*** Unknown tensor: ''',a,''' ***')
 1002 format('*** ',a1,' tensor set to ',a,' ***')
 1003 format('*** ',a1,' tensor converted to ',a,' ***')
 1011 format('gxx,gyy,gzz = ',2(f9.6,','),f9.6)
 1012 format('g1,g2,g3 = ',2(f9.6,','),f9.6)
 1013 format('gprp,grhm,gpll = ',2(f9.6,','),f9.6)
 1021 format('Axx,Ayy,Azz = ',2(f9.6,','),f9.6)
 1022 format('A1,A2,A3 = ',2(f9.6,','),f9.6)
 1023 format('Aprp,Arhm,Apll = ',2(f9.6,','),f9.6)
 1031 format('Rx,Ry,Rz = ',2(f9.6,','),f9.6)
 1032 format('Rbar,N,Nxy = ',2(f9.6,','),f9.6)
 1033 format('Rprp,Rrhm,Rpll = ',2(f9.6,','),f9.6)
      end
