c  VERSION 1.0  (NLSPR version)   4/7/93
c**********************************************************************
c
c                       ===================
c                       SUBROUTINE : FBASIS
c                       ===================
c     *** COMBINATION OF FBASO & FBASD ***
c
c          This routine builds a list of the basis set indices for
c     both diagonal and off-diagonal spaces in common block /indexf/
c     for the truncation parameters set in /eprprm/.
c          If the file, <fileid>.ind exists, it simply reads the basis
c     set.  This basis would be the pruned basis obtained by running
c     program eprbf.  If the file does not exist, it generates full
c     basis set within the given MTS parameters.
c
c     Includes:
c        nlsdim.inc
c        eprprm.inc
c        indexf.inc
c        stdio.inc
c
c     Uses:
c        ipar.f
c
c**********************************************************************
c
      subroutine fbasis(ixname,igen,ierr)
c
      implicit none
c
      include 'nlsdim.inc'
      include 'eprprm.inc'
      include 'indexf.inc'
      include 'stdio.inc'
c
      logical fexist
      character*30 ixname
      integer lu,igen,ierrtmp
c
      integer lr,jkr,kr,krmx,jmr,mr,mrmx,iper,iqer,iqermn,iqermx,
     #     ipnr,ipnrmx,ipnrmn,iqnr,iqnrmx,iqnrmn,nrow,i,j,ierr,ilm
c
      double precision one,sqrt2
c
      integer ipar
      external ipar
c
c######################################################################
c
      ierr=0
      ierrtmp=0
c
      if (igen.eq.1) go to 100
c
      inquire(file=ixname,exist=fexist)
      if (fexist) then
c
c----------------------------------------------------------------------
c     Read basis set information from index file
c----------------------------------------------------------------------
c
         open (unit=ludisk,file=ixname,status='old',
     #         access='sequential',form='unformatted')
         read (ludisk) ndimo
         read (ludisk) (pi1(i),qi1(i),l1(i),jk1(i),k1(i),
     #                  jm1(i),m1(i),i=1,ndimo)
         close (unit=ludisk)
         go to 200
      end if
      ierrtmp=1
c
c----------------------------------------------------------------------
c     Check the magnetic parameters
c----------------------------------------------------------------------
c
 100  call pcheck(luttyo,ierr)
c
      if (ierr.lt.0) then
         ierr=3
         return
      end if
c
      nrow=0
c
c----------------------------------------------------------------------
c     *** loop over lr ***
c----------------------------------------------------------------------
c
      do 110 lr=0,lemx,ldelta
        if((ipar(lr).ne.1).and.(lr.gt.lomx)) go to 110
c
c----------------------------------------------------------------------
c     *** loop over jkr ***
c----------------------------------------------------------------------
c
        do 120 jkr=jkmn,1,2
c
c----------------------------------------------------------------------
c     *** loop over kr ***
c----------------------------------------------------------------------
c
        krmx=min(kmx,lr)
        do 130 kr=0,krmx,kdelta
          if((kr.eq.0).and.(ipar(lr).ne.jkr)) go to 130
c
c----------------------------------------------------------------------
c     *** loop over jmr ***
c----------------------------------------------------------------------
c
          do 140 jmr=jmmn,1,2
c
c----------------------------------------------------------------------
c     *** loop over mr ***
c----------------------------------------------------------------------
c
            mrmx=min(mmx,lr)
            do 150 mr=0,mrmx
c
c----------------------------------------------------------------------
c     *** loop over ipnr ***
c----------------------------------------------------------------------
c
              ipnrmx=min(in2,ipnmx)
              if (mr.eq.0) then
                ipnrmn=0
              else
                ipnrmn=-ipnrmx
              end if                
c
              do 160 ipnr=ipnrmn,ipnrmx
                if((mr.eq.0).and.(ipnr.eq.0).and.(ipar(lr).ne.jmr))
     #                                 go to 160
                if((ipsi0.eq.0).and.(ipnr.ne.mr)) go to 160
c
c----------------------------------------------------------------------
c     *** loop over iqnr ***
c----------------------------------------------------------------------
c
                iqnrmx=in2-iabs(ipnr)
                iqnrmn=-iqnrmx
                do 170 iqnr=iqnrmn,iqnrmx,2
c
                  nrow=nrow+1
                  jqe1(nrow)=0
                  l1(nrow)=lr
                  jk1(nrow)=jkr
                  k1(nrow)=kr
                  jm1(nrow)=jmr
                  m1(nrow)=mr
                  pi1(nrow)=ipnr
                  qi1(nrow)=iqnr
c
c----------------------------------------------------------------------
c     end loop over rows
c----------------------------------------------------------------------
c
 170              continue
 160            continue
 150          continue
 140        continue
 130      continue
 120    continue
 110  continue
c
      ndimo=nrow
c
c**********************************************************************
c     Basis index for diagonal space (FBASD)
c**********************************************************************
c
 200  continue
c
      one=1.0D0
      sqrt2=dsqrt(2.0D0)
c
c----------------------------------------------------------------------
c       convert pruned off-diagonal basis set into diagonal one.
c----------------------------------------------------------------------
c
      j=0
      do 210 i=1,ndimo
         j=j+1
         dl1(j)=l1(i)
         djk1(j)=jk1(i)
         dk1(j)=k1(i)
         djm1(j)=0
         dm1(j)=m1(i)
         djqe1(j)=jm1(i)
         dpi1(j)=pi1(i)
         dqi1(j)=qi1(i)
         pid(i)=1
         if ((pi1(i).eq.0).and.(m1(i).eq.0)) then
            pp(j)=sqrt2
         else
            pp(j)=one
c     
            j=j+1
            dl1(j)=l1(i)
            djk1(j)=jk1(i)
            dk1(j)=k1(i)
            djm1(j)=0
            dm1(j)=-m1(i)
            djqe1(j)=jm1(i)
            dpi1(j)=-pi1(i)
            dqi1(j)=qi1(i)
            pid(i)=2
            ilm=ipar(l1(i)+m1(i))
            if (ilm.eq.jm1(i)) then
               pp(j)=one
            else
               pp(j)=-one
            end if
         end if
 210  continue
      ndimd=j
c
      if (ierrtmp.eq.1) ierr=1
      if ((ndimo.gt.mxdim).or.(diaflg.and.(ndimd.gt.mxdim))) ierr=2
c
      return
      end
