c  VERSION 1.0  (NLSPR version)   4/12/93
c*********************************************************************
c
c                       ==================
c                       SUBROUTINE : SIM2D
c                       ==================
c
c     Subroutine version of MOMD program
c     
c     This routine is intended for use with nonlinear least-squares
c     applications.  The routine calculates 2D-spectrum for the 
c     parameters given in /eprprm/ using the eigenvalues and eigenvectors
c     already obtained by evesa routine.
c
c     On Entry :
c
c        icalc : = 0  recalculate the spectrum using previous xspec.
c                     (valid when varying gib or lib parameters)
c                = 1  recalculate the spectrum using the previous
c                        eigenvalues and eigenvectors.
c                     (valid for series of 2D spectra with different
c                        experimental type, combination, mixing time
c                        and so on
c                      also valid when varying hwid parameter)
c                = 2  recalculate the eigenvalues and the spectrum
c        xspec : complex time-domain spectrum (used for icalc=0 case)
c
c     On Exit  :
c
c        xspec : complex time-domain spectrum (to be used with icalc=0)
c        spec  : magnitude frequency spectrum
c        ierr  : error flag
c                      = 0   normal return
c                      < 0   critical error in one of
c                            pcheck ; wrong parameter specification
c                            cqrt   ; Rutishauser QR-decomposition
c
c     Includes :
c               nlsdim.inc
c               stdio.inc
c               eprprm.inc
c               prmeqv.inc
c               parcom.inc
c               stvcom.inc
c               egvcom.inc
c
c     Uses :
c               evesa.f
c               spcalc.f
c               convft.f
c
c*********************************************************************
c
      subroutine sim2d(icalc,xspec,spec,npt1,npt2,nvar,
     #                 iflag,ixi,delx,ierr)
c
      implicit none
c
      include 'nlsdim.inc'
      include 'stdio.inc'
      include 'eprprm.inc'
      include 'prmeqv.inc'
      include 'parcom.inc'
      include 'stvcom.inc'
      include 'egvcom.inc'
c
      double precision zero,scal,pi
      complex*16 czero,ci
      parameter (zero=0.0D0,czero=(0.0D0,0.0D0),ci=(0.0d0,1.0d0))
c
      integer npt1,npt2,icalc,i,j,k,l,psval,iort,bdw,
     #        ntmp,nvar,iflag,ixi,ierr
      double precision t1,t2,cspsi,amin,amax,delx,spec(npt1,npt2)
c
      complex*16 xspec(npt1,npt2)
c
      logical evalOK
      external evalOK
c
c#####################################################################
c
      ierr=0
      pi=4.0d0*datan(1.0d0)
c
c----------------------------------------------------------------------
c     Load values from parameter array into /eprprm/
c----------------------------------------------------------------------
c
10    continue
c
      do 2 i=1,nfprm
         fepr(i)=fparm(i)
 2    continue
c
      do 4 i=1,niprm
         iepr(i)=iparm(i)
 4    continue
c
      if (icalc.ge.1) then
c                                    *** test validity of parameters
         call pcheck(luttyo,ierr)
         if (ierr.lt.0) then
            write(luout,1000)
            if (luout.ne.luttyo) write(luttyo,1000)
            return
         end if
c                                    *** off-diagonal starting vector
         call stveco
c
      end if
c
c---------------------------------------------------------------------
c     calculate eigenvalues & eigenvectors
c---------------------------------------------------------------------
c
      if (icalc.ge.2) then
c                        ===============================
c                     === loop over orientations (MOMD) ===
c                        ===============================
         do 100 iort=1,nort
c
            if (nort.gt.1) then
               cspsi=dfloat(iort-1)/dfloat(nort-1)
               psi=dacos(cspsi)*1.8D2/pi
            end if
c
            if (idebug.ne.0) then
               write (ludeb,2000) iort,psi
               write (ludeb,2001)
            end if
c                     *** solve for eigenvalues & vectors for pS=1
            call evesa(1,stvo,nevo,egvalx(1,iort),
     #                 egvecx(1,1,iort),npt1,ierr)
	    nev1(iort)=nevo
            do 105 i=1,nevo
 105           egvalx(i,iort)=egvalx(i,iort)+ci*b0
            if (ierr.ne.0) then
               write(luout,1011)
               if (luout.ne.luttyo) write (luttyo,1011)
               ierr=-abs(ierr)
               return
            end if
c
            if ( .not.evalOK(1,egvalx(1,iort),nvar,luout) ) then
c
c   ### This is a temporary fix for negative eigenvalues occuring  ###
c   ### in function evaluation.  If this problem happens in OFF-   ###
c   ### diagonal space, it USUALLY means the basis set being used  ###
c   ### is not properly pruned yet and the program issues an error ###
c   ### message and exit.  If the eigenvalues of the diagonal sub- ###
c   ### space have negative real part, the first component of the  ###
c   ### R tensor is increased by 1.0d-4 and the spectra are re-    ###
c   ### calculated.  This seems to be related with the pruning     ###
c   ### scheme for the diagonal basis and further development for  ###
c   ### a new scheme is necessary to fix this problem.             ###
c   ###                     July 19, 1993       Sanghyuk Lee       ###
c   ###                                                            ###
c
               write (luout,1012)
               if (luout.ne.luttyo) write (luttyo,1012)
               ierr=-1
               return
c
            end if
c                     *** solve for eigenvalues & vectors for pS=0
            if (diaflg.or.(iexp.gt.2)) then
c
               if (idebug.ne.0) write (ludeb,2011)
c
               call stvecd(egvalx(1,iort),egvecx(1,1,iort),npt1)
               call evesa(0,stvd,nevd,egvalz(1,iort),
     #                    egvecz(1,1,iort),npt1,ierr)
               nev0(iort)=nevd
               if (ierr.ne.0) then
                  write(luout,1011)
                  if (luout.ne.luttyo) write (luttyo,1011)
                  ierr=-abs(ierr)
                  return
               end if
c
               if ( .not.evalOK(0,egvalz(1,iort),nvar,luout) ) then
                  if (iflag.eq.1) then
                     fparm(IDX)=fparm(IDX)+1.0d-4
                     write (luout,1014)
                     if (luout.ne.luttyo) write (luttyo,1014)
                  else
                     write (luout,1016)
                     fparm(ixi)=fparm(ixi)+9.0d0*delx
                     delx=delx*10.0d0
                     if (luout.ne.luttyo) write (luttyo,1016)
                  end if
                  go to 10
               end if
c
            end if
c
 100     continue
c
      end if
c
c---------------------------------------------------------------------
c     calculate complex 2D spectrum in time domain
c---------------------------------------------------------------------
c
      if (icalc.ge.1) then
c
         do 200 j=1,npt2
         do 200 i=1,npt1
 200        xspec(i,j)=czero
c                        ===============================
c                     === loop over orientations (MOMD) ===
c                        ===============================
         do 210 iort=1,nort
            scal=1.0d0
            if ((nort.gt.1).and.((iort.eq.1).or.(iort.eq.nort)))
     #               scal=0.5d0
            nevo=nev1(iort)
            nevd=nev0(iort)
            call spcalc(scal,egvalx(1,iort),egvalz(1,iort),
     #          egvecx(1,1,iort),egvecz(1,1,iort),xspec,npt1,npt2)
 210     continue
c
      end if
c
c---------------------------------------------------------------------
c     Calculate magnitude 2D spectrum with Gaussian and/or Lorentzian
c     inhomogeneous Broadening
c---------------------------------------------------------------------
c
      call convft(xspec,spec,npt1,npt2)
c
      return
c
c=====================================================================
c     format statements
c=====================================================================
c
 1000 format(/5x,'*** SIM2D: Calculation attempted with illegal ',
     #       'parameters ***')
 1011 format(/5x,'*** Too many iterations in cqrt (Rutishauser) ***')
 1012 format(/5x,'*** Basis set with loose pruning criterion is ',
     #       'recommended ***')
 1014 format(/8x,'First diffusion component is increased by 1e-4'/)
 1016 format(/8x,'Forward step size for current Jacobian evaluation ',
     #       'is multiplied by 10'/)
c
 2000 format(/70('#'),/5x,'Orientation ',i2,'  : psi = ',f5.2,
     #       /70('#'))
 2001 format(/'### OFF-DIAGONAL ###')
 2011 format(/'### DIAGONAL ###')
c
c=====================================================================
c
      end
