c VERSION 1.1
c----------------------------------------------------------------------
c             I/O UTILITY ROUTINES FOR NLS COMMAND INTERPRETER
c
c  Contains the subroutines:
c       getlin  -- issue a prompt and retreive a line from command stream
c       gettkn  -- return a token from the given line
c       ungett  -- undo gettkn action
c       touppr  -- converts a string to uppercase
c       ftoken  -- returns a real number
c       itoken  -- returns an integer number
c----------------------------------------------------------------------


c----------------------------------------------------------------------
c                    =========================
c                       function GETLIN
c                    =========================
c----------------------------------------------------------------------
      function getlin( line )
      implicit none
      logical getlin
c
      include 'stdio.inc'
c
      integer ioerr
      character*80 line
c
      if(lucmd.eq.luttyi) call pprmpt
      read (lucmd,1001,iostat=ioerr) line
c
c   -- Echo line if required
c
      if (ioerr.eq.0 .and. lucmd.ne.luttyi .and. luecho.ne.0)
     #      write(luecho,1001) line
      if (luecho.ne.0 .and. luout.eq.lulog)
     #      write (lulog,1002) line
      getlin=ioerr.eq.0
      return
c
 1001 format(a)
 1002 format('>',a)
      end

c------------------------------------------------------------------------
c                         ===================
c                          subroutine GETTKN
c                         ===================
c 
c  Written for free-form input of parameters for slow-motional 
c  calculations.  Returns a token consisting of nonwhitespace
c  characters. Exceptions are '(', ')', '=', '*', and CR, which
c  are returned as single tokens.
c
c ------------------------------------------------------------------------
c
      subroutine gettkn(line,token,lth)
      implicit none
      integer lth
      character line*80,token*30,chr*1
c
      integer i,j,ichr,ichar
c
c *** Function definitions
c
c      logical white,is1tok,ispunc
c      white(chr) = chr.eq.' '.or. ichar(chr).eq.9
c      is1tok(chr) = chr.eq.'('.or.chr.eq.')'.or.chr.eq.'*'
c      ispunc(chr) = chr.eq.','.or.chr.eq.';'
c ***
c
c------------------------------------------
c  Find the next non-whitespace character 
c------------------------------------------
      i=0
    2 i=i+1
      chr=line(i:i)
      if ((chr.eq.' ' .or. ichar(chr).eq.9 .or. chr.eq.',' 
     #    .or. chr.eq.';') .and. i.lt.80) goto 2
c
      if (i.ge.80) then
        lth=0
        token=' '
        return
      end if
c
c-----------------------------------
c Check for single-character tokens 
c-----------------------------------
      if (chr.eq.'(' .or. chr.eq.')' .or. chr.eq.'*'
     #    .or. chr.eq.'=') then
        token=chr
        lth=1
        line=line(i+1:)
        return
      end if
c
c----------------------------------------------------------------
c Place the next continuous string of characters in the token
c (stop at whitespace, punctuation, and single-character tokens)
c----------------------------------------------------------------
      j=i
    3 j=j+1
      chr=line(j:j)
      if ( chr.eq.' '.or.ichar(chr).eq.9
     #   .or. chr.eq.'(' .or. chr.eq.')' .or. chr.eq.'*' 
     #   .or. chr.eq.'=' .or. chr.eq.',' .or. chr.eq.';' ) then
        token=line(i:j-1)
        lth=j-i
        line=line(j:)
        return
      else
       go to 3
      end if
      end
c----------------------------------------------------------------------
c                     =========================
c                       subroutine UNGETT
c                     =========================
c  Replaces given token at the beginning of the given line
c  (Oh for the string functions of C..)
c----------------------------------------------------------------------
      subroutine ungett(token,lth,line)
      implicit none
      character line*80,tmplin*80,token*30
      integer lth
      if (lth.gt.0.and.lth.lt.80) then
        tmplin=line
        line=token(:lth) // tmplin
      end if
      return
      end
      
c----------------------------------------------------------------------
c                    =========================
c                       subroutine TOUPPR
c                    =========================
c Converts string argument to uppercase
c----------------------------------------------------------------------

      subroutine touppr(string,lth)
      implicit none
      character string*30,chr
      integer i,ich,ichar,lth
c
c *** Function definition
c      logical islc
c      islc(chr) = ichar(chr).ge.97 .and. ichar(chr).le.122
c ***
c----------------------------------------------------------------------
c
      do 10 i=1,lth
        chr=string(i:i)
        ich=ichar(chr)
        if (ich.ge.97 .and. ich.le.122) string(i:i)=char( ich-32 )
   10   continue
      return
      end

c----------------------------------------------------------------------
c                    =========================
c                       subroutine FTOKEN
c                    =========================
c  Decodes a token into a floating point number
c----------------------------------------------------------------------
      function ftoken( token,lth,val )
      implicit none
      character token*30,tkn*30,tmptkn*30,chr*1
      integer i,ioerr,lth
      double precision val
      logical ftoken
c     
c *** Function definitions --- these don't work with every FORTRAN 
c     implementation.
c
c      logical isdot,isexp
c      isdot(chr)=chr.eq.'.'
c      isexp(chr)=(chr .eq. 'd' .or. chr .eq. 'D') .or.
c     1           (chr .eq. 'e' .or. chr .eq. 'E')
c ***
c----------------------------------------------------------------------
c
       tkn = token
       do 10 i=1,lth
        chr=tkn(i:i)
        if (chr.eq.'.') goto 11
        if (chr.eq.'d' .or. chr.eq.'D' .or. chr.eq.'e' .or. chr.eq.'E')
     #  then
          tmptkn=tkn(i:)
          tkn=tkn(:i-1) // '.' // tmptkn
          lth=lth+1
          go to 11
        end if
   10   continue
        tkn=tkn(:lth)//'.'
        lth=lth+1
 
 11     read(tkn,1000,err=13,iostat=ioerr) val
 12     ftoken=ioerr.eq.0
        return
c     
 13     ioerr=-1
        goto 12
c
 1000   format(bn,f20.10)
        end


c----------------------------------------------------------------------
c                    =========================
c                       function ITOKEN
c                    =========================
c----------------------------------------------------------------------
      function itoken( token,lth,ival )
      implicit none
      character*30 token
      integer lth,ival,ioerr
      logical itoken
c 
      read (token,1000,err=13,iostat=ioerr) ival
 12   itoken=ioerr.eq.0
      return
c
 13   ioerr=-1
      goto 12
c
 1000 format(bn,i20)
      end

